package cdox;

import cdox.gui.*;
import cdox.util.*;
import cdox.util.doc.*;
import java.util.*;
import java.util.prefs.*;
import java.io.*;
import java.net.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


/**
 * The main class.
 *@author Rutger Bezema, Andreas Schmitz
 */
public class CDox{

    /**
     * This is the documentation for CDox, used to display the helpbrowser (amongst other
     * things).
     */
    public static Documentation documentation=null;

    private static Localizer lang=null;
    private static CDox cdoxinstance=null;
    private CDoxFrame cdoxframe=null;
    private static String filename=null;
    // garbage collecting timer:
    private java.util.Timer timer;

    // used for the splash screen
    private static JWindow frontScreen;
    private static int imageWidth, imageHeight;
    private static Dimension wndSize;
    private int startx, starty;
    private JProgressBar proBar;
    private String progString;
    /** The Version. */
    public static final String VERSION="1.0.3";
    /**
     * This can be a prefix where to find the cdox.jar. Normally not needed when the
     * cdox.jar is in the current directory.
     */
    public static String prefixDir="";
    /**
     * This is the plugins directory. Default is plugins in the current directory.
     */
    public static String pluginsDir="plugins";

    /**
     * Inits the CDox.
     */
    public CDox(){
	documentation=null;
	lang=null;
	getLocalizer();
    }

    /**
     * Gets the Localizer for this application. This method will EXIT the Java VM if not even an
     * english language file has been found!
     *@return the Localizer for CDox. Note that only one Localizer is generated, no matter
     * how often this method is invoked.
     */
    public static Localizer getLocalizer(){
	if(lang==null){
	    try{
		lang=new Localizer(Locale.getDefault(),"/cdox/lang/");
	    }catch(IOException e){
		System.err.println("Failed to load your language, trying English as fallback...");
		try{
		    lang=new Localizer(Locale.ENGLISH,"/cdox/lang/");
		}catch(IOException ioe){
		    ErrorHandler.log(ioe,false);
		    System.err.println("Can't find ANY language file, broken installation?");
		    System.err.println("Please include above messages in your Bug-Report...");
		    System.exit(1);
		}
	    }
	}
	lang.provideKeystrokes(true);
	return lang;
    }

    // initializes the Documentation, called by the constructor.
    private void initDocumentation(CDoxFrame cFrame){
        if(documentation==null){
	    try{
	    documentation=new Documentation
		(cFrame, ClassLoader.getSystemResourceAsStream("cdox/cdox.xml"));
	    String f="cdox.jar";
	    if(!prefixDir.equals("")) f=prefixDir+"/"+f;
	    File file = new File(f);
	    URL url= null;
	    try{
		url = new URL("jar:"+file.toURL()+"!/cdox/images/cdoxhelp.jpg");
	    }catch(MalformedURLException murle){
		CDoxFrame.handleError(murle,true);
	    }
	    documentation.setBackgroundURL(url);
	    }catch(IOException ioe){
		CDoxFrame.handleError(ioe,true);
	    }
	}
    }

    /**
     * Pops up a dialog and asks for a language to choose. Returns the language which the
     * user chose as ISO3 language code (which can be placed in the preferences under
     * /cdox with key language) and displayable string in its own language.
     * @return a string array with the iso3-code in [0] and the display language in [1].
     */
    public static String[] askForLanguage(){
	Properties props=new Properties();
	try{
	    props.load(CDox.class.getResourceAsStream
		       ("/cdox/lang/availablelanguages"));
	}catch(IOException ioe){
	    ErrorHandler.log(ioe,false);
	    System.err.println("No language files found, sorry. Please include above message"+
			       " in your bug report.");
	}
	Object[] langs=props.keySet().toArray();
	class LangDialog extends JDialog implements ActionListener, WindowFocusListener{
	    String [] returnValues;
	    boolean okPressed;
	    Properties props;
	    Object[] langs;
	    JComboBox box;
	    public LangDialog(Object[] l, Properties p){
		props = p;
		langs = l;
		setResizable(false);
		getContentPane().setLayout(new GridBagLayout());
		setTitle("Choose Language");
		setModal(true);
		setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		GridBagConstraints gb = new GridBagConstraints();
		box = new JComboBox(langs);
		gb.insets = new Insets(5,2,15,2);
		getContentPane().add(box, gb);
		JButton but = new JButton("Use this language");
		but.addActionListener(this);
		gb.insets = new Insets(0,2,2,2);
		gb.anchor = gb.CENTER;
		gb.gridy = 2;
		gb.weighty = 3f;
		getContentPane().add(but, gb);
		pack();
		Dimension ss = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension fs = getSize();
		setLocation((ss.width-fs.width)/2,(ss.height-fs.height)/2);
		addWindowFocusListener(this);
		requestFocus();
	    }
	    public void actionPerformed(ActionEvent ae){
		int selected = box.getSelectedIndex();
		returnValues=new String[]{props.getProperty(langs[selected].toString()),langs[selected].toString()};
		hide();
	    }

	    public String[] getValues(){
		return returnValues;
	    }
	    public void windowLostFocus(WindowEvent we){
		requestFocus();
		toFront();
	    }
    	    public void windowGainedFocus(WindowEvent we){
	    }
	}
	LangDialog diag = new LangDialog(langs, props);
	diag.show();
	return diag.getValues();
    }

    // checks, if a language configuration has been done and asks for language, if in
    // doubt.
    private void checkLanguageSetting(){
	Preferences langsetting=Preferences.userNodeForPackage(getClass());
	String language=langsetting.get("language",null);
	if(language==null) language = askForLanguage()[0];
	lang.setLang("/cdox/lang/"+language);
	langsetting.put("language",language);
	try{
	    langsetting.flush();
	}catch(BackingStoreException bse){
	    CDoxFrame.handleError(bse,true);
	}
    }

    /**
     * It's here where the application starts.
     */
    public void start(){
	try{ // some nice ui
	    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
	} // should be changed...
	catch(ClassNotFoundException cnfe){
	    CDoxFrame.handleError(cnfe,true);
	}catch(InstantiationException ie){
	    CDoxFrame.handleError(ie,true);
	}catch(IllegalAccessException iae){
	    CDoxFrame.handleError(iae,true);
	}catch(UnsupportedLookAndFeelException ulafe){
	    CDoxFrame.handleError(ulafe,true);
	}
	checkLanguageSetting();
	Locale.setDefault(new Locale(lang.getLanguageCode().substring(0,2)));
	JComponent.setDefaultLocale(Locale.getDefault());

	progString = lang.get("startinit");
	initializeSplash();
// 	timer=new java.util.Timer(true);
// 	timer.schedule(new TimerTask(){
// 		public void run(){
// 		    System.gc();

// 		    System.out.println("Memory used: "+((Runtime.getRuntime().totalMemory()-
// 							Runtime.getRuntime().freeMemory())/1024)+"kb");
// 		    System.out.println("free: "+(Runtime.getRuntime().freeMemory()/1024)+"kb/total: "+
// 				       (Runtime.getRuntime().totalMemory()/1024)+"kb/max: "+
// 				       (Runtime.getRuntime().maxMemory()/1024)+"kb");
		   
// 		}
// 	    },5000,5000);
	if(filename!=null) cdoxframe=new CDoxFrame(this,filename);
	else cdoxframe=new CDoxFrame(this);
	initDocumentation(cdoxframe);
	cdoxframe.setVisible(true);
	cdoxframe.toFront();
	cdoxinstance=this;
	frontScreen.dispose();
	ErrorHandler.closeErrorLog();
	if(Preferences.userNodeForPackage(CDoxFrame.class).getBoolean("startwithwizard",true)){
	    cdoxframe.showNewWizard();
	    // update the size once again
	    if(!(cdoxframe.getExtendedState()==CDoxFrame.MAXIMIZED_BOTH))
		cdoxframe.center(cdoxframe);
	}
    }

    /**
     * This method should be invoked when CDox is stopped/restarted.
     * @return an <code>int</code> value indicating the JOptionPane option chosen.
     */
    public int stop(){
	int value = cdoxframe.checkCloseable();
	if(value == JOptionPane.CANCEL_OPTION){
	    return value;
	}
// 	timer.cancel();
// 	timer=null;
	System.out.println(lang.get("shutdown"));
	cdoxinstance=null;
	try{
	    File tmpDir=new File(System.getProperty("java.io.tmpdir"));
	    if(tmpDir.isDirectory()){
		File[] allTempFiles = tmpDir.listFiles();
		for(int i=0;i<allTempFiles.length;i++){

		    if(allTempFiles[i].isFile()){
			if(allTempFiles[i].getName().toLowerCase().startsWith("cdox")||
			   allTempFiles[i].getName().toLowerCase().startsWith("imageio")){
			    // System.out.println(allTempFiles[i].getName());			
			    allTempFiles[i].delete();
			}
		    }
		}
	    }
	}
	catch(Exception e){//SecurityException, NullPointEx, IllegalArgumentEx.
	    CDoxFrame.handleError(new Throwable("Delete it yourself"), false);
	}
	cdoxframe.hide();
	cdoxframe.dispose();
	cdoxframe=null;
	ErrorHandler.closeErrorLog();

	    
	    
	return -10;
    }

    /**
     *This method does all the work on the splash screen.
     */
    private void initializeSplash(){
	MediaTracker mt = new MediaTracker(frontScreen);
	frontScreen.getContentPane().setLayout(new BorderLayout());
	Image im = new ImageIcon(getClass().getResource("/cdox/images/cdoxsplash.jpg")).getImage();
	im = im.getScaledInstance(imageWidth, imageHeight, Image.SCALE_FAST);
	
	mt.addImage(im, 0);
	try{
	    mt.waitForID(0);
	}
	catch(InterruptedException ie){
	    CDoxFrame.handleError(ie,true);
	}
	startx = (wndSize.width/2)-(imageWidth/2);
	starty = (wndSize.height/2)-(imageHeight/2);
	frontScreen.setBounds(startx,starty , imageWidth, imageHeight+16);

	FrontScreenImage ipanel = new FrontScreenImage(im, new Dimension(imageWidth, imageHeight));
	frontScreen.getContentPane().add(ipanel,BorderLayout.CENTER);
	
	proBar = new JProgressBar(0, 100);
	frontScreen.getContentPane().add(proBar, BorderLayout.SOUTH);
	frontScreen.setVisible(true);
	frontScreen.toFront();
    }

    /**
     *This method sets the current progress it is called from the CDoxFrame constructor.
     *@param val the progress value in procents,
     *@param prog the progress String to draw on the splash screen.
     */
    public void setProgress(int val, String prog){
	progString = prog;
	frontScreen.repaint();
	proBar.setValue(val);
    }

    /**
     *This method returns the ideal height and size for this screen resolution.
     *@return the ideal Dimension for the splash/about image.
     */
    public static Dimension getImageDimension(){
	return new Dimension(imageWidth, imageHeight);
    }

    /**
     * Shuts the current running CDox down and starts a new one, re-reading the
     * configuration and those things.
     */
    public static void restartCDox(){
	cdoxinstance.stop();
	new CDox().startUp();
    }

    // starts it up. Maybe we should do this things in the constructor?
    private void startUp(){
	try{
	    ErrorHandler.setErrorFile("cdox.log");
	}catch(IOException ioe){
	    System.out.println("No log will be available: "+ioe.getLocalizedMessage());
	}
	frontScreen  = new JWindow();
	Toolkit theKit = frontScreen.getToolkit();
	wndSize = theKit.getScreenSize();
	imageWidth = (int)wndSize.width/4;
	imageHeight = (int)wndSize.height/5;
	start();
    }

    /**
     * Prints out usage information (possible command line arguments) on stdout and
     * exits.
     */
    public static void printUsage(){
	System.out.println(getLocalizer().get("usageinfo"));
	System.exit(0);
    }

    // yep, processes the command line arguments
    private static void processCmdlineArgs( String[] args ){
	int nr=args.length;
	int pos=0;
	StringBuffer sb = new StringBuffer();
	while(nr>pos){
	    if(args[pos].startsWith("-")&&(pos==(nr-1))) printUsage();
	    else if(args[pos].equals("-h")||args[pos].equals("--help")) printUsage();
	    else if(args[pos].equals("-dir")){
		pos++;
		prefixDir=args[pos];
		pos++;
		continue;
	    }else if(args[pos].equals("-plugins")){
		pos++;
		pluginsDir=args[pos];
		pos++;
		continue;
	    }else{
		//JDialog diag = new JDialog();
		sb.append(args[pos]);
		if(pos != nr)
		    sb.append(" ");
		//diag.setTitle(sb.toString());
		//diag.pack();
		//diag.show();
	    }
	    pos++;
	}
	filename = sb.toString().trim();
	if(filename.equals("")) filename=null;
    }

    /**
     * Run me.
     * @param args the arguments.
     * @exception IOException if an error occurs
     */
    public static void main( String[] args ) throws IOException{
	processCmdlineArgs(args);
	new CDox().startUp();
    }

    /**
     * This class draws an image on a JPanel.
     */
    private class FrontScreenImage extends JPanel{
	/**
	 *The constructor stores the given image.
	 *@param im is the Image to draw on the JPanel.
	 */
	public FrontScreenImage(Image im, Dimension dim){
	    image = im;
	    setPreferredSize(dim);
	    setMaximumSize(dim);
	    setMinimumSize(dim);
	}

	/**
	 *This method is called when the JPanel needs to be redrawn.
	 *It creates a Graphics2D object from the Graphics Object, and than redraws
	 *the image, with the text buzzer on it.
	 */
	public void paint(Graphics g){
	    Graphics2D g2d = (Graphics2D) g;
	    g2d.drawImage(image, 0, 0, this);
	    g2d.setPaint(Color.white);
	    g2d.drawString(progString,2, imageHeight-3);
	    g2d.setPaint(Color.BLACK);
	    g2d.drawString("Version "+VERSION,imageWidth-80,15);
	    frontScreen.toFront();
	}
	private Image image;
    }

}
