package cdox.data;

import cdox.edit.*;
import java.awt.*;
import org.w3c.dom.*;

/**
 *<p>
 * This interface is used to represent the data that is known about the current CD and to
 * fill this data on a cover.
 *</p>
 *<p>
 * The framework is meant to work together in the following way: the Data object is
 * independant from eventual data sources and does only store, well, the data about the
 * particular type of CD. The user can fill in parts of the data or the whole data
 * himself, and this data can be used together with a DataSource implementation to fill in
 * the rest of the data automatically (by querying a database or whatever).
 *</p>
 *<p>
 * A data object could also support a different style of data retrieval. If one has burned
 * a normal CD-R with some files, a Data object could let the user specify the path to the
 * CD-R, while a corresponding DataSource object would read the directories/files from the
 * path and fill the information into the Data object.
 *</p>
 *<p>
 * There exist no standard ways to fill data into a Data object. Since the DataSource
 * objects are always bound to a specific Data object it is up to the implementors to
 * provide useful methods for this purpose.
 *</p>
 *<p>
 * Another important method is the <code>getSupportedVersion()</code> method. It would be
 * quite bad, if the user uses a CDox/whatever that supports only version 1 of the file
 * format and downloads a plugin that uses features from version 2 of the file
 * format... Remember that the store data method fills in the data into the XML file, so
 * the wrong version would be bad. We'll try to keep the file format backwords compatible
 * of course.
 *</p>
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version June 22th 2002
 */
public interface Data{

    /**
     * This method should let the user fill in appropriate values as data.
     *@param parent the parent frame.
     *@return true if the user clicked ok, false otherwhise
     */
    public boolean userFillData( Frame parent );

    /**
     * Must return the version of the CDox file format that supports this data type.
     *@return the version number.
     */
    public int getSupportedVersion();

    /**
     * This method stores the data of this element in a DOM tree. The desc parameter is
     * the DOM element <code>&lt;description&gt;</code>. The type attribute of this
     * element must be set, and the child elements must be created according to the XML
     * Schema from the CDox format.
     *@param desc the <code>&lt;description&gt;</code> element.
     */
    public void storeData( org.w3c.dom.Element desc );

    /**
     * Returns a descriptive string of the CD type that this Data object represents.
     *@return the string.
     */
    public String getDescription();

    /**
     * Reads its data from the DOM subtree beginning with the given Element.
     *@param desc the top node of the DOM subtree.
     */
    public void reconstruct( org.w3c.dom.Element desc );

    /**
     * This method should fill the given cover with elements that are created from the
     * data which is stored here. This method is for front sides.
     *@param c the cover where to add new elements.
     *@throws IllegalArgumentException if the cover is no front side.
     */
    public void fillFront( Cover c ) throws IllegalArgumentException;

    /**
     * This method should fill the given cover with elements that are created from the
     * data which is stored here. This method is for back sides.
     *@param c the cover where to add new elements.
     *@throws IllegalArgumentException if the cover is no back side.
     */
    public void fillBack( Cover c ) throws IllegalArgumentException;

    /**
     * This method should fill the given cover with elements that are created from the
     * data which is stored here. This method is for booklets.
     *@param c the cover where to add new elements.
     *@throws IllegalArgumentException if the cover is no booklet.
     */
    public void fillBooklet( Cover c ) throws IllegalArgumentException;

}
