package cdox.data;

import java.awt.*;

/**
 *<p>
 * This interface is meant to provide easy access to a database, a specific file format or
 * any other data source that provides information about a CD.
 *</p>
 *<p>
 * The CDox program will use this classes typically the following way: it gets the various
 * description strings for all classes/plugins and displays them to the user (in the case
 * that he wants to fill in the values for the type of his CD automatically), if their
 * data type matches the current CD type. If the user chooses to configure a data source
 * (and the data source can be configured at all), the configuration dialog of the source
 * is shown. The user can also decide to perform the retrieval of the data, in which case
 * there is created a data object, normally by giving search information to it. That
 * object is then used to retrieve the data.
 *</p>
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 22nd 2002
 */
public interface DataSource{

    /**
     * Returns a short description for this source. This could be the String "CDDB
     * Database" for a CDDB query source and should be a String that tells the user
     * roughly what it can do.
     *@return the string.
     */
    public String getDescription();

    /**
     * Returns the class of data which this source provides. This is needed to determine
     * the CD type which suits the produced data.
     *@return the class.
     */
    public Class getDataType();

    /**
     * Used to determine whether there is a configuration dialog for this source.
     *@return true, if there is one.
     */
    public boolean hasConfigurationDialog();

    /**
     * Shows a configuration dialog where some options can be configured concerning this
     * source. This is no must for some sources, although quite useful. For example, a
     * CDDB source could show a dialog with the server to use, which port etc. The
     * configured options should be saved (for example in the preferences) by the
     * implementing classes.
     *@param parent the parent frame.
     */
    public void showConfigurationDialog( Frame parent );

    /**
     * This method will be invoked to get the data. The showConfigurationDialog() method
     * will be invoked before this method, if the hasConfigurationDialog() method returns
     * true. If there has been a dialog, the data could also have been retrieved there,
     * but it must be returned here. If this method returns null, it is assumed that
     * either something went wrong in the retrieval process or the user has cancelled the
     * dialog, so that the original data is not overwritten.
     * @return the data that is hopefully found.
     */
    public Data retrieveData();

}
