package cdox.data;

import cdox.*;
import cdox.gui.*;
import cdox.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

/**
 *<p>
 * This class provides access to the plugins that are currently installed.
 *</p>
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version October 11st 2002
 * @see Data
 * @see DataSource
 */
public class PluginFactory{

    /**
     * This method returns all DataSource classes that can be used to aquire the provided
     * Data class.
     *@param data a class that must be an instance of the Data interface.
     *@return all classes that match or null, if the parameter is no instance of Data.
     */
    public static Class[] getDataSourcesFor( Class data ){
	if(!Data.class.isAssignableFrom(data)) return null;
	Vector v=new Vector();
	if(data.equals(GenericData.class)){
	    v.add(FileSource.class);
	}
	if(data.equals(VideoData.class)){
	    v.add(VideoFileSource.class);
	}
	File dir=new File(CDox.pluginsDir);
	if(dir.isDirectory()){ // get each file,
	    File[] f=dir.listFiles();
	    for(int i=0;i<f.length;i++){
		if(f[i].getAbsolutePath().endsWith(".class")){ // check if ends with .class
		    try{ // load the class
			Class c=Class.forName(f[i].getName().substring
					      (0,f[i].getName().length()-6));
			// if it implements DataSource, add it
			if(DataSource.class.isAssignableFrom(c))
			    if(((DataSource)c.newInstance()).getDataType().equals(data)) v.add(c);
		    }catch(ClassNotFoundException cnfe){
			// message needed here?
		    }catch(InstantiationException ie){
			// message needed here?
		    }catch(IllegalAccessException iae){
			// message needed here?
		    }
		}
	    }
	}
	return (Class[])v.toArray(new Class[v.size()]);
    }

    /**
     * Returns all available data classes.
     *@return an array with all data classes.
     */
    public static Class[] getAvailableDataClasses(){
	Vector v=new Vector();
	// adding the generic ones
	v.add(AudioData.class);
	v.add(VideoData.class);
	v.add(GenericData.class);
	File dir=new File(CDox.pluginsDir);
	if(dir.isDirectory()){ // get each file,
	    File[] f=dir.listFiles();
	    for(int i=0;i<f.length;i++){
		if(f[i].getAbsolutePath().endsWith(".class")){ // check if ends with .class
		    try{ // load the class
			Class c=Class.forName(f[i].getName().substring
					      (0,f[i].getName().length()-6));
			// if it implements Data, add it
			if(Data.class.isAssignableFrom(c)) v.add(c);
		    }catch(ClassNotFoundException cnfe){
			// message needed here?
		    }
		}
	    }
	}
	return (Class[])v.toArray(new Class[v.size()]);
    }

    /**
     * Lets the user choose a data type for his CD.
     *@return the chosen Data or null, if none has been chosen.
     */
    public static Data chooseDataType( final Localizer lang ){
	// shows a dialog with the descriptions of all available data classes and lets the
	// user choose one.
	Class[] c=getAvailableDataClasses();
	Vector v=new Vector();
	final Data[] datas=new Data[c.length];
	try{
	    for(int i=0;i<c.length;i++){
		datas[i]=(Data)c[i].newInstance();
		v.add(datas[i].getDescription());
	    }
	}catch(InstantiationException ie){
	    CDoxFrame.handleError(ie,true);
	}catch(IllegalAccessException iae){
	    CDoxFrame.handleError(iae,true);
	}
	Chooser chooser=new Chooser(c,v,lang);
	chooser.show();
	try{
	    if(chooser.getChosen()==null) return null;
	    Data d=(Data)chooser.getChosen().newInstance();
	    return d;
	}catch(InstantiationException ie){
	    CDoxFrame.handleError(ie,true);
	}catch(IllegalAccessException iae){
	    CDoxFrame.handleError(iae,true);
	}
	return null;
    }

    /**
     * Lets the user choose a data source for his CD data.
     *@param lang a Localizer.
     *@param dClass the data class.
     *@return the chosen DataSource or null, if none has been chosen.
     */
    public static DataSource chooseDataSource( final Localizer lang, Class dClass ){
	// shows a dialog with the descriptions of all available data source classes and
	// lets the user choose one.
	Class[] c=getDataSourcesFor(dClass);
	if(c.length==0){
	    JOptionPane.showMessageDialog(null,lang.get("nosource"));
	    return null;
	}
	Vector v=new Vector();
	final DataSource[] datas=new DataSource[c.length];
	try{
	    for(int i=0;i<c.length;i++){
		datas[i]=(DataSource)c[i].newInstance();
		v.add(datas[i].getDescription());
	    }
	}catch(InstantiationException ie){
	    CDoxFrame.handleError(ie,true);
	}catch(IllegalAccessException iae){
	    CDoxFrame.handleError(iae,true);
	}
	Chooser chooser=new Chooser(c,v,lang);
	chooser.show();
	try{
	    if(chooser.getChosen()==null) return null;
	    DataSource d=(DataSource)chooser.getChosen().newInstance();
	    return d;
	}catch(InstantiationException ie){
	    CDoxFrame.handleError(ie,true);
	}catch(IllegalAccessException iae){
	    CDoxFrame.handleError(iae,true);
	}
	return null;
    }

    static class Chooser extends JDialog{

	private Class chosen=null;

	/**
	 * Constructs one.
	 *@param c the classes.
	 *@param d the descriptions.
	 */
	public Chooser( final Class[] c, Vector d, Localizer lang ){
	    setModal(true);
	    setTitle(lang.get("choosedata"));
	    final JList list=new JList(d);
	    getContentPane().setLayout(new GridBagLayout());
	    GridBagConstraints gb=new GridBagConstraints();
	    gb.gridx=0;
	    gb.gridy=0;
	    gb.fill=gb.BOTH;
	    gb.insets=new Insets(2,2,2,2);
	    gb.gridwidth=2;
	    getContentPane().add(list,gb);
	    gb.gridwidth=1;
	    gb.gridy=1;
	    JButton button;
	    getContentPane().add(button=new JButton(lang.get("okbutton")),gb);
	    button.addActionListener(new ActionListener(){
		    public void actionPerformed( ActionEvent evt ){
			try{
			    chosen=c[list.getSelectedIndex()];
			}catch(Exception e){
			    // chosen will be null, invalid choice
			}
			setVisible(false);
			dispose();
		    }
		});
	    getRootPane().setDefaultButton(button);
	    gb.gridx=1;
	    getContentPane().add(button=new JButton(lang.get("cancelbutton")),gb);
	    button.addActionListener(new ActionListener(){
		    public void actionPerformed( ActionEvent evt ){
			setVisible(false);
			dispose();
		    }
		});
	    new EscapeListener(button,getRootPane());
	    pack();
	    setLocationRelativeTo(null);
	    setResizable(false);
	}

	/**
	 * Returns the chosen Data.
	 *@return the data or null, if none has been chosen.
	 */
	public Class getChosen(){
	    return chosen;
	}

    }

    public static Data readData( org.w3c.dom.Element desc ){
	String type=desc.getAttributes().getNamedItem("type").getNodeValue();
	if(type==null) return null;
	Class[] classes=getAvailableDataClasses();
	Class c=null;
	for(int i=0;i<classes.length;i++)
	    if(classes[i].getName().endsWith(type)) c=classes[i];
	if(c==null) return null;
	try{
	    // if it implements Data, use it
	    if(Data.class.isAssignableFrom(c)){
		Data d=(Data)c.newInstance();
		d.reconstruct(desc);
		return d;
	    }
	}catch(InstantiationException ie){
	}catch(IllegalAccessException iae){
	}
	// none was found. Maybe message "Plugin not found"?
	return null;
    }

}
