package cdox.edit;

import cdox.data.*;
import cdox.gui.*;
import cdox.util.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;
import java.util.zip.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 *<p>
 * This is a class to load a CDox file from a stream/file whatever and to reconstruct the
 * Cover and Data objects that are stored within it. If the parse method is
 * invoked a second time, the previous stored Cover and Data objects will be lost.
 *</p>
 *<p>
 * A CDoxLoader can be used several times by simply calling the parse method. The
 * expected format of the parse method is of course a zip file format containing
 * the documents.cdx file, corresponding to the file format defined in the
 * fileformat.xsd. For more information, refer to the developer pages that can be found on
 * our <a href="http://cdox.sf.net">homepage</a>.
 *</p>
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 */
public class CDoxLoader implements CDCoverStandards{

    // these two will be available after parsing (the data object may be null)
    private Data data=null;
    private ArrayList covers=null;

    // various variables
    private ZipFile file=null;
    private DocumentBuilder db;

    /**
     * Constructs new loader.
     *@throws ParserConfigurationException if XML does not work correctly in the
     * implementation of the JRE/JDK/whatever.
     */
    public CDoxLoader() throws ParserConfigurationException{
	db=DocumentBuilderFactory.newInstance().newDocumentBuilder();
    }

    // resets some of the variables
    private void reset(){
	data=null;
	covers=null;
	file=null;
    }

    /**
     * Parses the file.
     *@param f the file to parse.
     *@throws IOException if the location is not readable.
     *@throws SAXException if the documents.cdx is no valid cdx XML file.
     */
    public void parse( File f ) throws IOException,SAXException{
	reset();
	covers=new ArrayList();
	Document doc=readFile(f);
	reconstruct(doc);
    }

    /**
     * Returns the parsed covers.
     *@return an array containing the covers.
     */
    public Cover[] getCovers(){
	return (Cover[])covers.toArray(new Cover[covers.size()]);
    }

    /**
     * Returns the parsed data or null, if none was found.
     *@return the data.
     */
    public Data getData(){
	return data;
    }

    // this one opens the zip file and extracts the documents.cdx, which is returned as
    // DOM document. The file variable will be set to the new ZipFile
    private Document readFile( File f ) throws IOException,SAXException{
	file=new ZipFile(f);
	ZipEntry d=file.getEntry("documents.cdx");
	if(d==null) return null;
	Document doc=db.parse(file.getInputStream(d));
	return doc;
    }

    // reconstructs the Cover and Data objects from the document
    private void reconstruct( Document doc ) throws IOException{
	// get the root element
	NodeList nl=doc.getElementsByTagName/*NS("http://cdox.sf.net/schema/fileformat",*/("cdox");
	org.w3c.dom.Element e=(org.w3c.dom.Element)nl.item(0);
	nl=e.getChildNodes();
	// iterate over all child nodes (description, cdfront etc.)
	for(int i=0;i<nl.getLength();i++){
	    Node n=nl.item(i);
	    if(n instanceof org.w3c.dom.Element){
		if(n.getNodeName().equals("description")) reconstructData((org.w3c.dom.Element)n);
		if(n.getNodeName().equals("cdfront")||
		   n.getNodeName().equals("cdbooklet")||
		   n.getNodeName().equals("cdback")) reconstructCover((org.w3c.dom.Element)n);
	    }
	}
    }

    // reconstructs the data object
    private void reconstructData( org.w3c.dom.Element desc ){
	data=PluginFactory.readData(desc);
    }

    // reconstructs a cover
    private void reconstructCover( org.w3c.dom.Element cov ) throws IOException{
	int type=-1;
	// determine the type of the cover
	if(cov.getNodeName().equals("cdfront")) type=CDFRONT;
	if(cov.getNodeName().equals("cdback")) type=CDBACK_SIDE;
	if(cov.getNodeName().equals("cdbooklet")) type=CDBOOKLET;
	// create it
	Cover c=new Cover(type);
	// check for background color
	String s=cov.getAttributes().getNamedItem("color").getNodeValue();
	if(s!=null) c.setBackground(new Color(Integer.parseInt(s)));
	// check for background image
	if(cov.getAttributes().getNamedItem("image")!=null)
	    s=cov.getAttributes().getNamedItem("image").getNodeValue();
	else s=null;
	if(s!=null){
	    boolean mode=true;
	    if(cov.getAttributes().getNamedItem("freesides")!=null)
		mode=cov.getAttributes().getNamedItem("freesides").getNodeValue().equals("false");
	    c.setBackground(TempFiles.add(file,file.getEntry(s)),mode);
	}
	// reconstruct elements
	NodeList nl=cov.getChildNodes();
	for(int i=0;i<nl.getLength();i++){
	    Node n=nl.item(i);
	    if(n instanceof org.w3c.dom.Element) reconstructElement(c,(org.w3c.dom.Element)n);
	}
	covers.add(c);
    }

    // reconstructs an element and adds it to the cover
    private void reconstructElement( Cover c, org.w3c.dom.Element elem ) throws IOException{
	NamedNodeMap a=elem.getAttributes();
	Point pos=new Point(Integer.parseInt(a.getNamedItem("positionx").getNodeValue()),
			    Integer.parseInt(a.getNamedItem("positiony").getNodeValue()));
	Dimension size=new Dimension(Integer.parseInt(a.getNamedItem("sizex").getNodeValue()),
				     Integer.parseInt(a.getNamedItem("sizey").getNodeValue()));
	double rotate=0;
	if(a.getNamedItem("rotate")!=null)
	    rotate=Double.parseDouble(a.getNamedItem("rotate").getNodeValue());
	boolean horiz=a.getNamedItem("fliphoriz").getNodeValue().equals("true");
	boolean vert=a.getNamedItem("flipvert").getNodeValue().equals("true");
	boolean aspect=a.getNamedItem("aspect").getNodeValue().equals("true");
	double aspectx=Double.parseDouble(a.getNamedItem("aspectx").getNodeValue());
	double aspecty=Double.parseDouble(a.getNamedItem("aspecty").getNodeValue());
	String type=a.getNamedItem("type").getNodeValue();
	NodeList nl=elem.getChildNodes();
	Element element=null;
	for(int i=0;i<nl.getLength();i++){
	    if(nl.item(i) instanceof org.w3c.dom.Element){
		if(type.equals("text")) element=reconstructTextElement((Point)pos.clone(),c,(org.w3c.dom.Element)nl.item(i));
		if(type.equals("image")) element=reconstructImageElement((Point)pos.clone(),size,(org.w3c.dom.Element)nl.item(i), c);
	    }
	}
	element.setMaintainRatio(aspect);
	element.setRatio(aspectx,aspecty);
	element.rotateElement(rotate);
	element.setSize(size);
	c.addElement(element);
	element.setPosition((Point)pos.clone());
	if(horiz) element.flipElementHorizontal();
	if(vert) element.flipElementVertical();
    }

    // returns a reconstructed text element
    private Element reconstructTextElement( Point pos, Cover c, org.w3c.dom.Element cont ){
	TextPane p=new TextPane(pos);
	//	p.addCaretListener(new CaretUpdater(cFrame, p));
	NodeList nl=cont.getElementsByTagName("text");
	for(int i=0;i<nl.getLength();i++){
	    NamedNodeMap atts=nl.item(i).getAttributes();
	    String bold=atts.getNamedItem("bold").getNodeValue();
	    String italic = atts.getNamedItem("italic").getNodeValue();
	    String underline = atts.getNamedItem("underline").getNodeValue();
	    String font = atts.getNamedItem("font").getNodeValue();
	    String color = atts.getNamedItem("color").getNodeValue();
	    String text = atts.getNamedItem("text").getNodeValue();
	    try{
		text=new String(Base64.decode(text.getBytes("utf8")),"utf8");
	    }catch(UnsupportedEncodingException uee){
		CDoxFrame.handleError(uee,true);
	    }
	    String height = atts.getNamedItem("height").getNodeValue();
	    p.appendText(bold, italic, underline,font,
			 color, text, height);
	}
	return new TextElement(pos,p);
    }

    // returns a reconstructed image element
    private Element reconstructImageElement( Point pos, Dimension size, org.w3c.dom.Element cont, Cover c ) throws IOException{
	org.w3c.dom.Element f=(org.w3c.dom.Element)cont.getElementsByTagName("file").item(0);
	return new ImageElement
	    (pos,size,TempFiles.add
	     (file,file.getEntry
	      (f.getAttributes().getNamedItem("name").getNodeValue())),c);
    }

}
