package cdox.edit;

import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.io.*;
import java.awt.datatransfer.*;
import java.util.zip.*;
import cdox.*;
import cdox.gui.*;
import cdox.util.*;
import cdox.gui.undo.*;

/**
 * This class represents an element that is contained within a document.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 *@version Jun 10th 2002
 */
public abstract class Element implements Transferable{

    // ************************************************************
    // general element properties
    /** The size of the element. */
    protected Dimension elementSize;
    /** The position. */
    protected Point position;
    /** The Cover. */
    protected Cover cover;
    /** Needed for the resizing position cursors. */
    protected Rectangle horizontalResizeRect = null,
	verticalResizeRect = null,
	diagonalResizeRect = null;
    /** The cached Element. If null, must be redrawn. */
    protected BufferedImage cachedImage=null;
    // borders that shall be drawn
    private boolean highLight=false;
    private boolean selected=false;
    // for the aspect ratio
    private boolean maintainRatio=true;
    // the next is to check whether the ratio has been changed already
    private boolean isMaintainingOriginalAspect=true;
    
    /**
     * The X ratio.
     */
    protected double ratioX=1;

    /**
     * The Y ratio.
     */
    protected double ratioY=1;

    // ************************************************************
    // some transform directions
    /** If true, the element will be transformed with highest quality. */
    protected boolean quality=false;
    /** Element will be rotated by this number as degrees. */
    protected double rotateBy=0;
    /** If true, element is flipped vertical. */
    protected boolean flipVertical=false;
    /** If true, element is flipped horizontal. */
    protected boolean flipHorizontal=false;

    // ************************************************************
    // border rotating stuff, static
    private static Cover rotateCover=null;
    /** This stroke is updated automatically by the timer. */
    protected static BasicStroke dashed=
	new BasicStroke(1.0f,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER, 
			5.0f, new float[]{5.0f,5.0f}, 0.0f);
    private static javax.swing.Timer rotateTimer=null;

    // ************************************************************
    // other
    private Localizer lang=CDox.getLocalizer();
    private AffineTransform printTransform = null;
    //private HashSet doactions=new HashSet(); // not needed yet (for multiple undo only)

    // ************************************************************
    // methods follow.
    // ************************************************************
    /**
     * Some general initialization work is done here.
     *@param size the size.
     *@param pos the position.
     */
    protected Element( Dimension size, Point pos ){
	// could run into problems, if pos/size incorrect (no problem if setCover is
	// called lateron)
	position=pos;
	if(size!=null){
	    elementSize=size;
	    ratioX=size.width;
	    ratioY=size.height;
	}
    }

    /**
     * Starts generating rotated borders of selected elements. Please be sure to restart
     * the rotating if the cover and/or the component are no longer used (otherwise you'll
     * end up in offscreen rotating. You want that? Sure?).
     *@param c the component whose repaint() method is frequently called.
     *@param element the element whose cover will be notified frequently about changes.
     */
    public static void startRotating( Component c, Element element ){
	if(rotateTimer!=null) stopRotating();
	rotateTimer=new javax.swing.Timer(600,new BorderRotator(c,element.cover));
	rotateTimer.start();
	rotateCover=element.cover;
    }

    /**
     * Returns whether we are rotating or not.
     *@return true, if we are.
     */
    public static boolean isRotating(){
	return (rotateTimer!=null);
    }

    /**
     * Stops the rotating border of selected elements. Just invoke this method if the
     * rotating is not needed any more.
     */
    public static void stopRotating(){
	if(rotateTimer==null) return;
	rotateTimer.stop();
	rotateTimer=null;
	rotateCover.setSelectedElement(null);
	rotateCover.fireChanged();
	rotateCover=null;
    }

    /**
     * Sets the quality.
     *@param q if true, high quality will be used.
     */
    public void setQuality( boolean q ){
	cachedImage=null;
	quality=q;
    }


    /**
     * Sets, if this element is selected. Selected elements get another rectangle drawed
     * around them.
     *@param selected whether this element should be selected.
     */
    public void setSelected( boolean selected ){
	if(selected) cover.setSelectedElement(this);
	this.selected=selected;
	cover.fireChanged();
    }

    /**
     *This Methods returns true if the Element is selected.
     *@return true if the Element is selected.
     */
    public boolean isSelected(){
	return selected;
    }

    /**
     * Toggles highlighting of this element. If true, causes a rectancle be drawn around this
     * Element, default is false.
     */
    public void setHighlight(){
	highLight = !highLight;
	cover.fireChanged();
    }
    
    /**
     * Returns whether this Element is highlighted or not.
     *@return true, if the element is highlighted.
     */
    public boolean getHighlight(){
	return highLight;
    }

    /**
     * Sets the cover that owns this element.
     *@param c the cover owning this element. Note that only one cover can use this
     * element at the same time.
     */
    protected void setCover( Cover c ){
	this.cover=c;
	setPosition(position);
	setSize(elementSize);
    }

    /**
     * Returns true if the element is at the specified position.
     *@param p the position.
     *@return true, if d is a point within the elements position.
     */
    public boolean isInPlace( Point p){
	return (((p.x<(elementSize.getWidth()+position.x))&&
		(p.y<(elementSize.getHeight()+position.y)))&&
		((p.x>position.x))&&
		(p.y>position.y));
    }

    /**
     * Moves the element to the top of the list.
     */
    public void moveToTop(){
	cover.moveToTop(this);
    }

    /**
     * Sets whether the aspect ratio of this element should be maintained or not. Default
     * is true.
     *@param b if true, the ratio will be maintained.
     */
    public void setMaintainRatio( boolean b ){
	maintainRatio=b;
	if(!b) isMaintainingOriginalAspect=false;
    }

    /**
     * Returns whether this element does maintain its ratio.
     *@return true, if it does.
     */
    public boolean isMaintainingRatio(){
	return maintainRatio;
    }

    /**
     * Sets the ratio that should be used, if any.
     *@param x the x aspect.
     *@param y the y aspect.
     */
    public void setRatio( double x, double y ){
	ratioX=x;
	ratioY=y;
    }

    /**
     * Sets the new size of the element. This is the final size of the <i>element</i>, not
     * of the image/text whatsoever. If the size won't fit into the cover, the size is
     * made smaller.
     *@param s the new size.
     */
    public void setSize( Dimension s ){
	s=(Dimension)s.clone();
	if(cover!=null){
	    if(cover.size[0]<s.width) s.width=cover.size[0];
	    if(cover.size[1]<s.height) s.height=cover.size[1];
	    if(isMaintainingRatio()){
		// well, works now. Dunno why...
		if(ratioX>ratioY){// whether width or height should be greater
		    // if width should be greater, the given width is multiplied with the
		    // ratio Y/X (which is <1). Result is the correct height.
		    s.height=(int)(((double)s.width)*(ratioY/ratioX));
		    // there is one case missing:
		    if(s.height>cover.size[1]){
			// in that case, do it backwards, make the width smaller so the height
			// fits onto the cover
			s.width=(int)(cover.size[1]*(ratioX/ratioY));
			s.height=cover.size[1];
		    }
		}else{
		    // if height should be greater, the given height is multiplied with the
		    // ration X/Y (which is <1). Result is the correct width.
		    s.width=(int)(((double)s.height)*(ratioX/ratioY));
		    // there is one case missing: (untested, probably this can't happen with
		    // the standard sizes)
		    if(s.width>cover.size[0]){
			// in that case, do it backwards, make the height smaller so the width
			// fits onto the cover
			s.height=(int)(cover.size[0]*(ratioY/ratioX));
			s.width=cover.size[0];
		    }
		}
	    }else{
		ratioX=s.width;
		ratioY=s.height;
	    }
	}
	this.elementSize=s;
	setResizeRects();
	cachedImage=null;
	checkSizeAndPosition();
    }

    /**
     * Sets the new position of the element. Negative values will be counted as 0.
     *@param pos the new position.
     */
    public void setPosition( Point pos ){
	pos=(Point)pos.clone();
	if(pos.x<0) pos.x=0;
	if(pos.y<0) pos.y=0;
	position=pos;
	checkSizeAndPosition();
	setResizeRects();
	if(pos.x<0) pos.x=0;
	if(pos.y<0) pos.y=0;
    }

    // checks if we are out of bounds and does something about it. Does not check, whether
    // the element fits into the cover with its size, this is checked in setsize, negative
    // positions are checked in setposition
    private void checkSizeAndPosition(){
	if(cover==null) return;
	if(cover.size[0]<(elementSize.width+position.x))
	    position.x=cover.size[0]-elementSize.width;
	if(cover.size[1]<(elementSize.height+position.y))
	    position.y=cover.size[1]-elementSize.height;
    }

    /**
     * Returns the size of this element.
     *@return the size.
     */
    public Dimension getSize(){
	return elementSize;
    }
    
    /**
     * Returns the upper left corner of the Element.
     *@return the the upper left corner.
     */
    public Point getPosition(){
	return position;
    }

    /**
     * Sets the resizing rectangles.
     */
    protected void setResizeRects(){
 	horizontalResizeRect =
	    new Rectangle(position.x+(elementSize.width-3), position.y, elementSize.width,
			  elementSize.height-5);
 	verticalResizeRect =
	    new Rectangle(position.x, position.y+(elementSize.height-3), elementSize.width-5,
			  elementSize.height);
 	diagonalResizeRect =
	    new Rectangle(position.x+(elementSize.width-5),position.y+(elementSize.height-5),
			  elementSize.width, elementSize.height);
    }

    /**
     * Returns the horizontal resize rectangle.
     *
     * @return a <code>Rectangle</code> value.
     */
    public Rectangle getHorizontalResizeRect(){
 	return horizontalResizeRect;
    }

    /**
     * Returns the vertical resize rectangle.
     *
     * @return a <code>Rectangle</code> value.
     */
    public Rectangle getVerticalResizeRect(){
 	return verticalResizeRect;
    }

    /**
     * Returns the diagonal resize rectangle.
     *
     * @return a <code>Rectangle</code> value.
     */
    public Rectangle getDiagonalResizeRect(){
 	return diagonalResizeRect;
    }

    /**
     * Flips the element vertical.
     */
    public void flipElementVertical(){
	cachedImage=null;
	flipVertical= !flipVertical;
    }


    /**
     * Flips the element horizontal.
     */
    public void flipElementHorizontal(){
	cachedImage=null;
	flipHorizontal = !flipHorizontal;
    }

    // skipping this since not needed yet.
    /*
     * This method invalidates the DoActions which are assigned to this element, causing
     * them typically to do nothing on redo/undo invocation.
     */
    //public void invalidateDoActions(){
	//Iterator i=doactions.iterator();
	//	while(i.hasNext()) ((DoAction)i.next()).invalidate();
	// (might be needed for undo/redo lists)
    //    }

    /**
     * Registers a doaction with this element. Does currently nothing.
     * @param da a <code>DoAction</code> value.
     */
    public void registerDoAction( DoAction da ){
	//doactions.add(da);
    }

    /**
     * Unregisters a doaction with this element. Does currently nothing.
     * @param da a <code>DoAction</code> value.
     */
    public void unregisterDoAction( DoAction da ){
	//doactions.remove(da);
    }

    /**
     * Rotates the element by the given degrees.
     *@param degrees the degrees (0-360).
     */
    public void rotateElement( double degrees ){
	cachedImage=null;
	rotateBy=(rotateBy+degrees)%360;
	BufferedImage img=getBufferedImage(); // get the image
	// the first op is to check the needed translation and new ratio
	AffineTransformOp op=new AffineTransformOp(getMatrix(),null);
	if(isMaintainingOriginalAspect){ // this one is from the original image
	    ratioX=op.getBounds2D(img).getWidth();
	    ratioY=op.getBounds2D(img).getHeight();
	    int max=elementSize.width;
	    if(max<elementSize.height) max=elementSize.height;
	    setSize(new Dimension(max,max));
	}else{ // determines the aspect of the scaled image
	    AffineTransform at=new AffineTransform();
	    at.scale(((double)elementSize.width)/((double)img.getWidth()),
		     ((double)elementSize.height)/((double)img.getHeight()));
	    AffineTransformOp op2=new AffineTransformOp(at,null);
	    ratioX=op2.getBounds2D(img).getWidth();
	    ratioY=op2.getBounds2D(img).getHeight();
	    int max=elementSize.width;
	    if(max<elementSize.height) max=elementSize.height;
	    setSize(new Dimension(max,max));
	}
   }

    /**
     * This method creates the matrix that is needed to perform the various
     * transformations such as rotating/flipping. The returned does not do the position
     * translating, this is done while drawing.
     *@return the AffineTransform that contains the matrix.
     */
    protected AffineTransform getMatrix(){
	// the /real/ order of transform is "do it backwards". Yes, rotating comes first,
	// then flipping etc. Took awhile, to find it out. No, I don't have that Linear
	// Algebra paper yet.
	AffineTransform at=new AffineTransform();
	if(flipHorizontal) at.scale(1,-1);
	if(flipVertical) at.scale(-1,1);
	if(rotateBy!=0) at.rotate(Math.toRadians(rotateBy));
	return at;
    }

    /**
     * Draws itself on the provided graphics object.
     *@param g where you like it to be drawn.
     */
    protected void drawMyself( Graphics2D g ){
	if(cachedImage==null) createCachedImage();
	AffineTransform  at=AffineTransform.getTranslateInstance(position.x,position.y);
	g.drawRenderedImage(cachedImage,(at.getDeterminant()==0)?null:at);
	drawBorders(g); // borders cannot be cached
    }

    // just draws the darn borders onto the graphics object
    private void drawBorders( Graphics2D g ){
	if(highLight&&(!quality)){
	    g.setPaint(Color.magenta);
	    g.drawRect(position.x-1,position.y-1,elementSize.width+1,elementSize.height+1);
	}
	if(selected&&(!quality)){
	    g.setPaint(Color.white);
	    g.drawRect(position.x,position.y,elementSize.width-1,elementSize.height-1);
	    Stroke s=g.getStroke();
	    g.setStroke(dashed);
	    g.setPaint(Color.black);
	    g.drawRect(position.x,position.y,elementSize.width-1,elementSize.height-1);
	    g.setStroke(s);
	}
    }

    /**
     * This method must return an <i>untransformed</i> image of the element. The various
     * transformations are done on this element, so it is a good idea to base the
     * size of this image on the size of the element to achieve best quality.
     *@return the BufferedImage.
     */
    protected abstract BufferedImage getBufferedImage();

    /**
     * Creates an image of the element. This performs all needed transformation in the
     * following order: rotating, flipping, translating (to correct the
     * rotating/flipping) and scaling. Position translation is a task of drawing and not
     * done here.
     */
    protected void createCachedImage(){
	try{
	    BufferedImage img=getBufferedImage(); // get the image
	    // the first op is to check the needed translation and new ratio
	    AffineTransformOp op=new AffineTransformOp(getMatrix(),null);
	    if(isMaintainingOriginalAspect){ // this one is from the original image
		ratioX=op.getBounds2D(img).getWidth();
		ratioY=op.getBounds2D(img).getHeight();
		setSize(elementSize);
	    }else{ // determines the aspect of the scaled image
		AffineTransform at=new AffineTransform();
		at.scale(((double)elementSize.width)/((double)img.getWidth()),
			 ((double)elementSize.height)/((double)img.getHeight()));
		AffineTransformOp op2=new AffineTransformOp(at,null);
		ratioX=op2.getBounds2D(img).getWidth();
		ratioY=op2.getBounds2D(img).getHeight();
		setSize(elementSize);
	    }
	    AffineTransform at=new AffineTransform(); // the final transform, must be started
	    // backwards: first scaling
	    at.scale(((double)elementSize.width)/((double)op.getBounds2D(img).getWidth()),
		     ((double)elementSize.height)/((double)op.getBounds2D(img).getHeight()));
	    // then translating
	    at.translate(-op.getBounds2D(img).getX(),-op.getBounds2D(img).getY());
	    at.concatenate(getMatrix()); // then the other stuff
	    if(at.getDeterminant()==0)
		at=new AffineTransform();
	    setPrintAffineTransform(at);
	    op=new AffineTransformOp(at,cover.getRenderingHints()); // and finally;
	    cachedImage=op.filter(img,new BufferedImage // do it
				  ((int)op.getBounds2D(img).getWidth(),
				   (int)op.getBounds2D(img).getHeight(),
				   BufferedImage.TYPE_INT_ARGB));
	}catch(ImagingOpException ioe){
	    CDoxFrame.handleError(ioe,true);
	}
    }
    
    private void setPrintAffineTransform(AffineTransform at){
	printTransform = new AffineTransform(at);
    }

    /**
     * Returns the affine transform used for printing.
     *
     * @return an <code>AffineTransform</code> value.
     */
    public AffineTransform getPrintableAffineTransform(){
	return new AffineTransform(printTransform);
    }


    /**
     * Appends an "element" element to the root-element with all required attributes
     * set.
     *@param root the Element to which the child will be appended.
     *@param type the type of the element which will be added (currently one of "text" or
     * "image").
     *@return the newly created element, to which the content element has to be added.
     */
    protected org.w3c.dom.Element appendChild( org.w3c.dom.Element root, String type ){
	org.w3c.dom.Element child=root.getOwnerDocument().createElementNS
	    ("http://cdox.sf.net/schema/fileformat","element");
	child.setAttribute("positionx",position.x+"");
	child.setAttribute("positiony",position.y+"");
	child.setAttribute("sizex",elementSize.width+"");
	child.setAttribute("sizey",elementSize.height+"");
	child.setAttribute("fliphoriz",flipHorizontal+"");
	child.setAttribute("flipvert",flipVertical+"");
	child.setAttribute("aspect",maintainRatio+"");
	child.setAttribute("aspectx",ratioX+"");
	child.setAttribute("aspecty",ratioY+"");
	child.setAttribute("type",type);
	if(rotateBy!=0) child.setAttribute("rotate",rotateBy+"");
	return (org.w3c.dom.Element)root.appendChild(child);
    }

    /**
     * This method saves its data in the given parameters. It appends a child element
     * ("element") to the given root element and saves its file(s), if any, to the
     * ZipOutputStream. There exists a convenience method (appendChild()) which appends
     * the child containing all required attributes to the root element, while returning
     * the newly created child, so you can concentrate on the content (which must be
     * appended to the "element" element).
     *@param root the root element to append the child to.
     *@param out the ZipOutputStream to write any files to.
     *@throws IOException if writing into the ZipStream failed.
     *@see #appendChild
     */
    protected abstract void saveMyself( org.w3c.dom.Element root, ZipOutputStream out )
	throws IOException;

    public String toString(){
	if(this instanceof ImageElement) return lang.get("imageelement");
	if(this instanceof GenericElement) return lang.get("genericelement");
	if(this instanceof TextElement) return lang.get("textelement");
	return "OtherElement? This is a bug, mailto cdox@gmx.net";
    }

    /**
     * This method can be invoked to indicate that this Element is no longer needed.
     */
    public void destroyMe(){
	cover=null;
    }

}
