package cdox.edit;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.io.*;
import javax.swing.*;
import java.awt.datatransfer.*;
import java.util.*;
import java.util.zip.*;

/**
 * This class represents an element that is contained within a document. The
 * GenericElement can contain a java.awt.Component as data.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 */
public class TextElement extends Element{

    private TextPane component=null;
    private Vector attr;
    private ArrayList realFontSizes = new ArrayList(), largeFontSizes=new ArrayList();

    private final int procent = 30;


    /**
     * The DataFlavor for cut/pasting text elements.
     */
    public static DataFlavor TEXTELEMENT_FLAVOR=null;
    static{ // he-he, didn't know about this one
	if(TEXTELEMENT_FLAVOR==null){
	    TEXTELEMENT_FLAVOR=new DataFlavor
		(TextElement.class,"Text element");
	}
    }


    /**
     * Creates new element at the specified position.
     * @param pos the position.
     * @param c the component aka the content.
     * @see TextPane
     */
    public TextElement( Point pos, TextPane c){
	super(c.getPreferredSize(),pos);
	component=c;
	component.setBackground(new Color(0f,0f,0f,0f));
	component.setBorder(null);
	c.getFontSizes(realFontSizes);
 	setResizeRects();
	c.setCaretPosition(0); // no selection any more in a text element
	c.setCaretColor(new Color(0f, 0f, 0f, 0f));
	setSize(c.getPreferredSize());
    }

    /**
     *This Method overrides the Element setSize method to enlarge the text.Elements in
     *this textPane, for better rendering.
     *@param dim the new Dimension.
     *@see TextPane#resizeRelativeAttributeSet
     *@see Element#setSize
     */
    public void setSize(Dimension dim){
	//just resize when enlarged
	int compArea=component.getPreferredSize().width*component.getPreferredSize().height;
	int elementArea = elementSize.width*elementSize.height;
	while(compArea<(6*elementArea)){
	    component.resizeRelativeAttributeSet(procent);
	    compArea = component.getPreferredSize().width*component.getPreferredSize().height;
	}
	//Thread.currentThread().dumpStack();
	super.setSize(dim);
	cachedImage = null;
    }


    /**
     *Resizes all the text-Elements to its original fontsize. And clears the 
     *realFontSizes Array.
     */
    public void resetFontSizes(){
	component.resetFontSizes(realFontSizes);
	realFontSizes.clear();
    }

    /**
     * Returns the TextPane for this text element.
     *@return the pane.
     */
    public TextPane getTextPane(){
	return component;
    }

    public DataFlavor[] getTransferDataFlavors(){
	return new DataFlavor[]{TEXTELEMENT_FLAVOR};
    }

    public boolean isDataFlavorSupported( DataFlavor f ){
	return (f.equals(TEXTELEMENT_FLAVOR));
    }

    public Object getTransferData( DataFlavor f ) throws UnsupportedFlavorException{
	if(!f.equals(TEXTELEMENT_FLAVOR)) throw new UnsupportedFlavorException(f);
	TextPane pane=new TextPane(component);
	TextElement t=new TextElement((Point)position.clone(),pane);
	return t;
    }

    protected BufferedImage getBufferedImage(){
	Dimension s=(Dimension)component.getPreferredSize().clone();
	BufferedImage img=new BufferedImage
	    (s.width,s.height,BufferedImage.TYPE_INT_ARGB);
	Graphics2D g=(Graphics2D)img.createGraphics();
 	SwingUtilities.paintComponent
 	    (g,component,new Container(),0,0,s.width,s.height);
	// huh, hah, finally found the bug: getPreferredSize returns half the height after
	// calling paintComponent the first time. So if the size is wrong, just call it a
	// second time. Could run into trouble while calling this recursively, though.
	Dimension d=component.getPreferredSize();
	if((d.width!=s.width)||(d.height!=s.height)) return getBufferedImage();
	return img;
    }

    protected void saveMyself( org.w3c.dom.Element root, ZipOutputStream out )
	throws IOException{
	org.w3c.dom.Element elem=appendChild(root,"text");
	org.w3c.dom.Element cont=elem.getOwnerDocument().createElementNS
	    ("http://cdox.sf.net/schema/fileformat","content");
	resetFontSizes();
	component.saveMyself(cont); // adds all those <text> tags
	elem.appendChild(cont);
    }

}
