package cdox.edit;

import cdox.*;
import cdox.util.*;
import java.awt.*;
import javax.swing.*;

/**
 * This class provides some useful methods for generation of text elements. See the
 * description of the various static methods how they work.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version July 5th 2002
 */
public class TextElementGenerator implements CDCoverStandards{

    private static final Font defaultFont=new JLabel().getFont();

    /**
     *<p>
     * Creates a text element from a single string. The element will be centered on the
     * cover according to the following rules:
     *</p>
     *<ul>
     *<li>If the cover is a front, simply center it.</li>
     *<li>If the cover is a back side, center it, but restrict the size to
     * CDBACK_SIDE_SIZE[0]-CDSIDE_SIZE[0]*2.</li>
     *<li>If the cover is a booklet: center it on the left or the right side. If the
     * parameter "left" is true, take the left side, if it is false, take the right.</li>
     *</ul>
     *<p>
     * If the string is empty, this method inserts no element at all. The font may be
     * null, in which case new JLabel().getFont() will be used. If the size is lower than
     * 6, 10 will be used.
     *</p>
     *@param c the cover where the element will be added.
     *@param s the string.
     *@param f the desired font.
     *@param size the desired font size.
     *@param yPos the y starting position.
     *@param left the mode (only relevant with booklets, see above).
     *@param bold whether the text should be bold.
     *@param italic whether the text should be italic.
     *@param underline whether the text should be underlined.
     *@return the text element that has already been added to the cover.
     */
    public static TextElement createSingleString( Cover c, String s, Font f, int size, int yPos,
						  boolean left, boolean bold, boolean italic,
						  boolean underline ){
	// exit on invalid parameters:
	if(c==null) return null;
	if(s==null) return null;
	if(s.equals("")) return null;
	// set default values, if invalid:
	if(size<6) size=10;
	if(f==null) f=defaultFont;
	// do the work:
	TextPane p=new TextPane(new Point());
	p.setAttribute(f,size,Color.BLACK,bold,italic,underline);
	p.setText(s);
	TextElement e;
	c.addElement(e=new TextElement(new Point(),p));
	// calculate size and position for the various types:
	Point pos=new Point(0,yPos);
	switch(c.getType()){
	case CDBACK_SIDE:
	    if(e.getSize().getWidth()>(CDBACK_SIDE_SIZE[0]-2*CDSIDE_SIZE[0]))
		e.setSize(new Dimension(CDBACK_SIDE_SIZE[0]-CDSIDE_SIZE[0]*2-2,
					CDBACK_SIDE_SIZE[0]-CDSIDE_SIZE[0]*2-2));
	    pos.x=(int)(CDBACK_SIDE_SIZE[0]-2*CDSIDE_SIZE[0]-e.getSize().getWidth())/2;
	    if(pos.x<=CDSIDE_SIZE[0]) pos.x=CDSIDE_SIZE[0]+1;
	    break;
	case CDBOOKLET:
	    if(e.getSize().getWidth()>(CDBOOKLET_SIZE[0]/2))
		e.setSize(new Dimension(CDBOOKLET_SIZE[0]/2-2,CDBOOKLET_SIZE[0]/2-2));
	    pos.x=(int)(CDBOOKLET_SIZE[0]/2-e.getSize().getWidth())/2;
	    if(pos.x==0) pos.x=1;
	    if(!left) pos.x+=CDBOOKLET_SIZE[0]/2;
	    break;
	case CDFRONT:
	    pos.x=(int)(CDFRONT_SIZE[0]-e.getSize().getWidth())/2;
	    if(pos.x==0) pos.x=1;
	    break;
	}
	e.setPosition(pos);
	return e;
    }

    /**
     *<p>
     * Creates a text element from multiple strings. The elements will be centered on the
     * cover according to the following rules:
     *</p>
     *<ul>
     *<li>If the cover is a front, simply center it.</li>
     *<li>If the cover is a back side, center it, but restrict the size to
     * CDBACK_SIDE_SIZE[0]-CDSIDE_SIZE[0]*2.</li>
     *<li>If the cover is a booklet: center it on the left or the right side. If the
     * parameter "left" is true, take the left side, if it is false, take the right.</li>
     *</ul>
     *<p>
     * If the strings are empty, this method inserts no element at all. The font may be
     * null, in which case new JLabel().getFont() will be used. If the size is lower than
     * 6, 10 will be used.
     *</p>
     *@param c the cover where the element will be added.
     *@param s the strings.
     *@param f the desired font.
     *@param size the desired font size.
     *@param yPos the y starting position.
     *@param left the mode (only relevant with booklets, see above).
     *@param bold whether the text should be bold.
     *@param italic whether the text should be italic.
     *@param underline whether the text should be underlined.
     *@return the text elements that have already been added to the cover.
     */
     public static TextElement[] createMultipleStrings( Cover c, String[] s, Font f, int size, int yPos,
 						  boolean left, boolean bold, boolean italic,
 						  boolean underline ){
	// exit on invalid parameters:
	if(c==null) return null;
	if(s==null) return null;
	if(s.equals("")) return null;
	// set default values, if invalid:
	if(size<6) size=10;
	if(f==null) f=defaultFont;
	// do the work:
	// calculate base values about column size
	String[] ready=null;
	// the columns are the count of characters that fit in a line
	int columns=0;
	double column=(CDBACK_SIDE_SIZE[0]-CDSIDE_SIZE[0]*2)/60;
	switch(c.getType()){
	case CDBOOKLET: columns=(int)((CDBOOKLET_SIZE[0]/2)/column);
	    break;
	case CDFRONT: columns=(int)(CDFRONT_SIZE[0]/column);
	    break;
	case CDBACK_SIDE: columns=(int)((CDBACK_SIDE_SIZE[0]-CDSIDE_SIZE[0]*2)/column);
	    break;
	}
	// create the strings that will be used in the text elements, check for multiple
	// columns
	if(s.length>25){
	    if(JOptionPane.YES_OPTION==JOptionPane.showConfirmDialog
	       (null,CDox.getLocalizer().get("wantmorecols",new String[]{s.length+""}))){
		ready=new String[s.length/25+1];
		int maxCols=columns/ready.length;
		for(int i=0;i<ready.length;i++){
		    StringBuffer sb=new StringBuffer();
		    for(int j=0;j<25;j++)
			if((j+i*25)<s.length){
			    if(s[j+i*25].length()>maxCols)
				s[j+i*25]=s[j+i*25].substring(0,maxCols-4)+"...";
			    sb.append(s[j+i*25]);
			    if(((j+i*25)!=s.length-1)&&(j!=24)) sb.append("\n");
			}
		    ready[i]=sb.toString();
		}
	    }else{
		StringBuffer sb=new StringBuffer();
		for(int i=0;i<s.length;i++){
		    sb.append(s[i]);
		    if(i!=(s.length-1)) sb.append("\n");
		}
		ready=new String[1];
		ready[0]=sb.toString();
	    }
	}else{
	    StringBuffer sb=new StringBuffer();
	    for(int i=0;i<s.length;i++){
		sb.append(s[i]);
		if(i!=(s.length-1)) sb.append("\n");
	    }
	    ready=new String[1];
	    ready[0]=sb.toString();
	}
	TextElement[] elements=new TextElement[ready.length];
	for(int i=0;i<ready.length;i++){
	    //	    if(ready[i].endsWith("\n")) ready[i]=ready[i].substring(0,ready[i].length()-1);
	    TextPane p=new TextPane(new Point());
	    p.setAttribute(f,size,Color.BLACK,bold,italic,underline);
	    p.setText(ready[i]);
	    TextElement e;
	    c.addElement(e=new TextElement(new Point(),p));
	    // calculate size and position for the various types:
	    Point pos=new Point(0,yPos);
	    switch(c.getType()){
	    case CDBACK_SIDE:
		if(e.getSize().getWidth()>(CDBACK_SIDE_SIZE[0]-2*CDSIDE_SIZE[0]))
		    e.setSize(new Dimension(CDBACK_SIDE_SIZE[0]-CDSIDE_SIZE[0]*2-2,
					    CDBACK_SIDE_SIZE[0]-CDSIDE_SIZE[0]*2-2));
		pos.x=CDSIDE_SIZE[0]+(i*((CDBACK_SIDE_SIZE[0]-2*CDSIDE_SIZE[0])/ready.length))+1;
		if(ready.length==1){
		    pos.x=(CDBACK_SIDE_SIZE[0]-e.getSize().width)/2;
		    if(pos.x<CDSIDE_SIZE[0]) pos.x=CDSIDE_SIZE[0]+1;
		}
		break;
	    case CDBOOKLET:
		if(e.getSize().getWidth()>(CDBOOKLET_SIZE[0]/2))
		    e.setSize(new Dimension(CDBOOKLET_SIZE[0]/2-2,CDBOOKLET_SIZE[0]/2-2));
		pos.x=i*((CDBOOKLET_SIZE[0]/2)/ready.length)+1;
		if(ready.length==1){
		    pos.x=(CDBOOKLET_SIZE[0]/2-e.getSize().width)/2;
		    if(pos.x==0) pos.x=1;
		}
		if(!left) pos.x+=CDBOOKLET_SIZE[0]/2;
		break;
	    case CDFRONT:
		pos.x=i*((CDBOOKLET_SIZE[0]/2)/ready.length)+1;
		break;
	    }
	    e.setPosition(pos);
	    elements[i]=e;
	}
	return elements;
     }

    /**
     * This method creates two text elements as side strings.
     *@param c the Cover. Must be a CDBACK_SIDE.
     *@param s the string to add.
     */
    public static void createSideString( Cover c, String s ){
	// exit on invalid parameters:
	if(c==null) return;
	if(c.getType()!=CDBACK_SIDE) return;
	if(s==null) return;
	if(s.equals("")) return;
	// do the work:
	TextPane p=new TextPane(new Point());
	p.setAttribute(new JLabel().getFont(),18,Color.BLACK,false,false,false);
	p.setText(s);
	TextElement e;
	c.addElement(e=new TextElement(new Point(),p));
	Dimension d=new Dimension(CDSIDE_SIZE[0],e.getSize().width);
	e.rotateElement(270);
	e.setMaintainRatio(false);
	e.setSize(d);
	e.setMaintainRatio(true);
	e.setPosition(new Point(0,(CDBACK_SIDE_SIZE[1]-e.getSize().height)/2));
	p=new TextPane(new Point());
	p.setAttribute(new JLabel().getFont(),18,Color.BLACK,false,false,false);
	p.setText(s);
	c.addElement(e=new TextElement(new Point(),p));
	d=new Dimension(CDSIDE_SIZE[0],e.getSize().width);
	e.rotateElement(90);
	e.setMaintainRatio(false);
	e.setSize(d);
	e.setMaintainRatio(true);
	e.setPosition(new Point(CDBACK_SIDE_SIZE[0]-CDSIDE_SIZE[0],(CDBACK_SIDE_SIZE[1]-e.getSize().height)/2));
    }

}
