package cdox.gui;

import cdox.*;
import cdox.edit.*;
import cdox.gui.undo.*;
import cdox.util.*;
import cdox.util.image.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

/**
 * A dialog which let the user slide a JSlider to correct a given BufferedImage. The dialog
 * consist of two ImagePreview panes (the original and the preview), an indicator that lets
 * the user know the filtering is done, a JSlider (or more than one) and two buttons. The
 * user will be able to filter his/her images.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version 25 april 2002.
 * @see ImagePreview
 * @see StatusDrawer
 * @see CorrectorSlider
 */
public class ImageDialog extends JDialog implements Observer, ActionListener, Runnable{

    private BufferedImage theImage;
    private ImageElement element;
    private Cover cover;
    private Localizer lang=CDox.getLocalizer();
    private ImagePreview original, vorschau;
    private StatusDrawer status;
    private CDoxFrame cFrame;
    private ArrayList usedImageCorrectors = new ArrayList(3);
    
    /**
     * This constant represents the BRIGHTNESS_CONTRAST_GAMMA edit-image-menu-item in the
     * popup-menu.
     */
    public static final int BRIGHTNESS_CONTRAST_GAMMA = 0;
    /**
     * This constant represents the SHARPEN edit-image-menu-item in the popup-menu.
     */
    public static final int SHARPEN = 1;
    /**
     * This constant represents the BLUR edit-image-menu-item in the popup-menu.
     */
    public static final int BLUR = 2;
    
    /**
     * Creates a new a new ImageDialog with the BufferedImage and the key representing
     * filters to be available. Depending on the key the dialog will have one or three
     * sliders to be used. This method should not be called. Instead call one of the other
     * Constructors.
     * @param parent the Frame which is the parent of this dialog
     * @param bi the bufferedImage to be filtered
     * @param key the key representing the filters which are available
     */
    protected ImageDialog( Frame parent, BufferedImage bi, int key){
	super(parent,true);
	getContentPane().setLayout(new GridBagLayout());
	setDefaultCloseOperation(EXIT_ON_CLOSE);
	cFrame = (CDoxFrame)parent;
	setTitle(lang.get("imgdlgtitle"));
	theImage = bi;
	//the panels
	JPanel schauPane = new JPanel(new GridBagLayout());
	GridBagConstraints gb = new GridBagConstraints();
	JPanel origPane = new JPanel();
	origPane.setBorder(BorderFactory.createCompoundBorder
			   (BorderFactory.createTitledBorder
			    (BorderFactory.createLineBorder(Color.black), 
			     lang.get("imgoriginal"),TitledBorder.LEADING,TitledBorder.ABOVE_TOP),
			    BorderFactory.createLoweredBevelBorder()));
	original = new ImagePreview(theImage);
	origPane.add(original);
	JPanel vorschauPane = new JPanel();
	vorschauPane.setBorder(BorderFactory.createCompoundBorder
			       (BorderFactory.createTitledBorder
				(BorderFactory.createLineBorder(Color.black), 
				 lang.get("imgpreview"),TitledBorder.LEADING,TitledBorder.ABOVE_TOP),
				BorderFactory.createLoweredBevelBorder()));
	vorschau = new ImagePreview(theImage);
	vorschauPane.add(vorschau);

	//the status drawer
	status = new StatusDrawer(this);
	//adding to the schauPane
	gb.insets = new Insets(3,3,3,3);
	schauPane.add(origPane, gb);
	gb.gridx = 1;
	schauPane.add(status, gb);
	gb.gridx = 2;
	schauPane.add(vorschauPane, gb);
	gb.gridx = 0; //Constraints back to default;
	gb.insets = new Insets(0,0,0,0);
	
	JPanel sliderPane = new JPanel(new GridBagLayout());
	CorrectorSlider imgC= null;
	switch(key){
	case BRIGHTNESS_CONTRAST_GAMMA:
	    for(int i = 0 ;i<3;i++){
		gb.gridy=i;
		switch (i){
		case 0:
		    imgC=new Brightness(this, lang.get("imgbrightnessslider"));
		    break;
		case 1:
		    imgC=new Contrast(this, lang.get("imgcontrastslider"));
		    break;
		case 2:
		    imgC=new GammaCorrection(this, lang.get("imggammaslider"));
		    break;
		}
		sliderPane.add(imgC.getSliderPane(), gb);
	    }
	    break;
	case SHARPEN:
	    imgC= new Sharpen(this, lang.get("imgsharpenslider"));
	    sliderPane.add(imgC.getSliderPane(), gb);
	    break;
	case BLUR:
	    imgC= new Blur(this, lang.get("imgsoftenslider"));
	    sliderPane.add(imgC.getSliderPane(), gb);
	    break;
	}
	gb.gridy = 0;//Constraints back to default;
	
	JPanel buttonPane = new JPanel(new GridBagLayout());
	JButton okButton = new JButton(lang.get("okbutton"));
	okButton.addActionListener(this);
	JButton cancelButton = new JButton(lang.get("cancelbutton"));
	new EscapeListener(cancelButton, getRootPane());
	getRootPane().setDefaultButton(okButton);
	cancelButton.addActionListener(this);
	gb.weightx = 1.0;
	buttonPane.add(okButton, gb);
	gb.gridx = 1;
	gb.weightx = 0;
	buttonPane.add(cancelButton, gb);
	gb.gridx = 0;//Constraints back to default;

	getContentPane().add(schauPane, gb);
	gb.insets= new Insets(5,0,5,0);
	gb.gridy=1;
	getContentPane().add(sliderPane, gb);
	gb.insets=new Insets(0,0,0,0);
	gb.gridy=2;
	getContentPane().add(buttonPane, gb);
	setResizable(false);

	((CDoxFrame)parent).center(this);
	show();
    }

    /**
     * Creates a new a new ImageDialog with the BufferedImage and the key representing
     * filters to be available. Depending on the key the dialog will have one or three
     * sliders to be used. This constructor is called when a backgroundImage must be
     * filtered.
     * @param parent the Frame which is the parent of this dialog
     * @param bi the bufferedImage to be filtered
     * @param c The cover which contains the backgroundImage to be filtered.
     * @param key the key representing the filters which are available
     * @see Cover
     */
    public ImageDialog( Frame parent, BufferedImage bi, Cover c, int key){
	this(parent, bi, key);
	cover = c;

    }
    /**
     * Creates a new a new ImageDialog with the ImageElement and the key representing the
     * filters to be available. Depending on the key the dialog will have one or three
     * sliders to be used. This constructor is called when an ImageElement must be
     * filtered.
     * @param parent the Frame which is the parent of this dialog
     * @param e the ImageElement to work on.
     * @param key the key representing the filters which are available
     * @see ImageElement
     */
    public ImageDialog( Frame parent, ImageElement e, int key){
	this(parent, e.getCurrentImage(), key);
	element = e;
    }

    /**
     * Is called when either one of the buttons is pressed.
     * If the OK-Button is pressed a new Thread is started adding all the filters to the
     * original Image.
     * @param e the event representing a buttonclick.
     * @see #run
     */
    public void actionPerformed(ActionEvent e){
	JButton button = (JButton)e.getSource();
	if(button.getText().equals(lang.get("okbutton"))){
	    new Thread(this, "end").start();
	}
	else
	    cFrame.setWorking(false);
	dispose();
    }


    /**
     * Is called when a imageCorrection slider has been moved. It sets the preview. All
     * used filters are stored in an ArrayList, and are applied to the preview one at a
     * time.
     * @param obs the ImageCorrection class which's slider has been moved.
     * @param arg the Filter to use.
     */
    public void update(Observable obs, Object arg){
	status.setWorking(true);
	ImageCorrector tempCor = (ImageCorrector)arg;
	if((!(usedImageCorrectors.contains(tempCor)))){
	    usedImageCorrectors.add(tempCor);
	}
	new Thread(this, "preview").start();
    }

    /**
     *This run Method does all the filtering of an image. It needs to be done <I>in a
     *seperate thread</I>, because the application will seem to hang if not. And if the
     *ok-button is pressed the dialog seems to stay.
     *@see #update
     *@see #actionPerformed
     */
    public void run(){
	if((Thread.currentThread()).getName().equals("preview")){
	    BufferedImage tempImage = vorschau.getOriginalImage();
	    for(int i=0; i<usedImageCorrectors.size();i++){
		tempImage=((ImageCorrector)usedImageCorrectors.get(i)).filter(tempImage, null);
	    }
	    vorschau.setPaintImage(tempImage);
	    vorschau.repaint();
	    status.setWorking(false);
	    status.repaint();
	}
	else if ((Thread.currentThread()).getName().equals("end")){
	    for(int i = 0; i<usedImageCorrectors.size();i++){
		theImage=((ImageCorrector)usedImageCorrectors.get(i)).filter(theImage, null);
	    }
	    if(element != null){
		String oldKey = element.getKey();
		element.setPaintImage(theImage);
		element.resetImage();
		String newKey = element.getKey();
		if(!oldKey.equals(newKey)){
		    cFrame.setUndoAction(new FotoEditAction(cFrame,element,oldKey, newKey)); 
		}

	    }
	    else if(cover != null){
		String oldKey = cover.getKey();
		cover.setFilteredBackgroundImage(theImage);
		cover.saveMemory();
		String newKey = cover.getKey();
		if(!oldKey.equals(newKey)){
		    cFrame.setUndoAction(new BackgroundEditAction(cFrame,cover,oldKey,newKey)); 
		}
	    }
	    cFrame.paintComponents(cFrame.getGraphics());
	    cFrame.setWorking(false);
	    
	}   
    }
}
