package cdox.gui;

import cdox.*;
import cdox.edit.*;
import cdox.gui.action.*;
import cdox.util.*;
import cdox.util.image.*;
import java.awt.image.*;
import java.awt.*;
import java.io.*;
import java.util.prefs.*;
import java.util.zip.*;
import javax.imageio.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * <p>This class represents a FileChooser with a preview pane, which previews the current
 * selected File, if it is an Image-File typ, and if the user chooses to do so with the
 * supplied checkbox. This Class has four FileFilters set: <ul>
 * <li>jpg</li>
 * <li>png</li>
 * <li>gif</li>
 * <li>cdx</li></ul>.</p>
 * <p>This Filechooser uses a Preferences Object to get the last visited directory, and sets
 * the current directory to this one, if no directory is set the users home-directory is
 * used. This fileChooser can be used to save or open a file.</p>
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version April 18th 2002.
 * @see ExtensionFilter 
 * @see ImagePreview
 */

public class MyFileChooser extends JFileChooser implements CDCoverStandards , ChangeListener{
    //the last visited directory
    private Preferences prefs=Preferences.userNodeForPackage(getClass());
    //the extensionfilters
    private ExtensionFilter pngFilter, gifFilter, jpgFilter, cdxFilter;
    //the previewpane
    private ImagePreview previewPane;
    //the previewPane container
    private JPanel previewContainer;
    //boolean if to preview or not
    private boolean preview;
    //checkbox to preview or not
    private JCheckBox previewCheckBox;
    

    /**
     *Constructs a new MyFilechooser with an added PreviewPane in it. It sets the fileFilters,
     *and checks if the workingdir exists. If so the directory is set to this, if not the
     *users home-directory is set. The user can only select one file at the Time. In the
     *previewpane (which displays the selected foto) a checkbox is added. It can be
     *selected for preview, or deselected for no preview. The checkbox is registered with
     *this class as the ChangeListener.
     *@see #stateChanged
     */
    public MyFileChooser(){
	String wDir = prefs.get("workingdir", System.getProperties().getProperty("user.dir"));
	File dirFile = new File(wDir);
	if(!dirFile.exists()) dirFile=new File(System.getProperty("user.dir"));
	setCurrentDirectory(dirFile);

	//for the preview and its checkbox
	int checkBoxSelected = prefs.getInt("previewselect", 1); //0 = false, 1 = true;
	if (checkBoxSelected != 1)
	    preview = false;
	else
	    preview = true;
	//the container.
	previewContainer = new JPanel(new GridBagLayout());
	GridBagConstraints gb = new GridBagConstraints();
	previewPane = new ImagePreview(null);
	previewCheckBox = new JCheckBox(CDox.getLocalizer().get("imgpreview"), preview);
	previewCheckBox.addChangeListener(this);
	previewContainer.add(previewPane, gb);
	gb.gridy=1;
	gb.anchor=gb.SOUTHWEST;
	gb.weighty=100;
	previewContainer.add(previewCheckBox, gb);
	setAccessory(previewContainer);

	pngFilter = new ExtensionFilter(".png", "PNG-Files (*.png)", null);
	gifFilter = new ExtensionFilter(".gif", "Gif-Files (*.gif)", null);
	jpgFilter = new ExtensionFilter(".jpg", "Jpg-Files (*.jpg, *.jpeg)", ".jpeg");
	cdxFilter = new ExtensionFilter(".cdx", "CDox-Files (*.cdx)", null);
	setAcceptAllFileFilterUsed(false);
	setMultiSelectionEnabled(false);
	setFileSelectionMode(FILES_ONLY);
    }

    /**
     *This method overwrites the JFileChooser's setSelectedFile method. It is called when
     *the user selects a File, or "ups" a directory in which case the File is null. This
     *method checks if the selected File is an image File by making a BufferedImage of it,
     *if it is not null, it is given to the previewpane <I>(if the user wants a preview)</I>
     *which will display it. After that the super is called.
     *@param f the current selected File
     *@see ImagePreview#setNewPreviewImage
     */
    public void setSelectedFile(File f){
	if(f != null){
	    if(preview){
		BufferedImage previewImage = null;
		try{
		    ZipFile zipFile = new ZipFile(f);
		    if(zipFile !=null){
			ZipEntry zipEntry = zipFile.getEntry("coverPreview_image.png");
			if(zipEntry !=null){
			    previewImage = ImageIO.read(zipFile.getInputStream(zipEntry));
			    previewPane.setNewPreviewImage(previewImage);//and preview it
			    previewPane.repaint();
			}
		    }
		}
		catch(ZipException ze){//if the file is not a zipFile
		    try{
			previewImage = ImageIO.read(f);//read the file
			previewPane.setNewPreviewImage(previewImage);//and preview it
			previewPane.repaint();
		    }
		    catch(IOException ioe){
			previewPane.setNewPreviewImage(null);
			previewPane.repaint();
			super.setSelectedFile(f);
		    }
		    super.setSelectedFile(f);
		}
		catch(IOException ioe){ 
		    previewPane.setNewPreviewImage(null);
		    previewPane.repaint();
		    super.setSelectedFile(f);
		}
	    }
	    else{
		previewPane.setNewPreviewImage(null);
	    }
	    previewPane.repaint();
	}
	else{
	    previewPane.setNewPreviewImage(null);
	    previewPane.repaint();		
	}
	super.setSelectedFile(f);    
    }
    /**
     * This method is called when the user wants a preview or not. Selected is preview. It
     * than calls the setSelectedFile to update the previewPane.
     * @param ce the event made by the checkbox.
     */
    public void stateChanged(ChangeEvent ce){
	if(((JCheckBox)ce.getSource()).isSelected())
	    preview =true;
	else{
	    preview =false;
	}
	setSelectedFile(getSelectedFile());
    }
    
    /**
     * This Method is setting some options to the MyFileChooser getFiles and than returns
     *the selected file (if any). First it gets the last used extensionfilter, it than
     *adds the filters to this dialog <I>the filters selected depends on the usage of this
     *filechooser, defined in FileAction.actionPerformed</I> sets the last selected one,
     *than waits for the approve button. If the approve button is clicked the current
     *settings are saved. This method also arranges the ending of a saved file.
     * @param cFrame the parent frame.
     * @param dialogTitle sets the Title of the JFilecChooser
     * @param approveButtonText sets the "approval" buttons Text
     * @param file the current file if any
     * @param filterDefiner the filters to be used for this MyFileChooser, save has
     * different fileFilters as open...etc.
     * @return the selected file, or null if none selected.
     * @see FileAction#actionPerformed
     */
    public File showDialog(JFrame cFrame, String dialogTitle, String approveButtonText, File file, int[] filterDefiner){
	setDialogTitle(dialogTitle);
	setApproveButtonText(approveButtonText);
	rescanCurrentDirectory();
	setSelectedFile(file);
	int lastFileFilterUsed = prefs.getInt("extensionfilter",ALL);
	boolean lastFilterValid=false;
	//setting the FileFilters and the last selected FileFilter as currently active
	for (int i = 0; i<filterDefiner.length;i++){
	    switch (filterDefiner[i]){
	    case ALL:
		setAcceptAllFileFilterUsed(true);
		if(lastFileFilterUsed==ALL) lastFilterValid=true;
		break;		
	    case JPG:
		addChoosableFileFilter(jpgFilter);
		if(lastFileFilterUsed==JPG) lastFilterValid=true;
		break;
	    case GIF:
		addChoosableFileFilter(gifFilter);
		if(lastFileFilterUsed==GIF) lastFilterValid=true;
		break;
	    case PNG:
		addChoosableFileFilter(pngFilter);
		if(lastFileFilterUsed==PNG) lastFilterValid=true;
		break;
	    case CDX:
		addChoosableFileFilter(cdxFilter);
		if(lastFileFilterUsed==CDX) lastFilterValid=true;
		break;		
	    }
	}
	if(lastFilterValid){
	    switch(lastFileFilterUsed){
	    case JPG:
		setFileFilter(jpgFilter);
		break;
	    case GIF:
		setFileFilter(gifFilter);
		break;
	    case PNG:
		setFileFilter(pngFilter);
		break;
	    case CDX:
		setFileFilter(cdxFilter);
		break;
	    case ALL:
		setAcceptAllFileFilterUsed(true);
		break;
	    }
	}

	//If the approve button is pressed the "previewselect-on/off", "current directory"
	//and "FileFilter" are saved.
	int result = showDialog(cFrame, null);
	File returnFile = null;
	if(result == APPROVE_OPTION){
	    prefs.putInt("previewselect", ((preview)? 1:0));
	    prefs.put("workingdir", getCurrentDirectory().getPath());
	    String h=getFileFilter().toString().substring(0);
	    if(h.endsWith("gif")) prefs.putInt("extensionfilter",GIF);
	    else if(h.endsWith("png")) prefs.putInt("extensionfilter",PNG);
	    else if(h.endsWith("jpg")||h.endsWith("jpeg")) prefs.putInt("extensionfilter",JPG);
	    else if(h.endsWith("cdx"))prefs.putInt("extensionfilter",CDX);
	    else prefs.putInt("extensionfilter",ALL);
	    try{
		prefs.flush();
	    }catch(BackingStoreException bse){//insider....
		CDoxFrame.handleError(bse,true);
	    }
	    //we get the selected file here
	    returnFile = getSelectedFile();
	    /**
	     *This if-clause sets the ending of the file to be saved, if the user didn't
	     *gave one. For example the user wants to save the file as a jpg-mime-type,
	     *and types only myPicture, this if-clause rearranges the file so that it is
	     *myPicture.jpg .
	     */
	    if((getFileFilter() instanceof ExtensionFilter)&&
	       (approveButtonText.equals(CDox.getLocalizer().get("fileapprovesavebutton")))){ 
		String extension = ((ExtensionFilter)getFileFilter()).getExtension();
		String alternativ = ((ExtensionFilter)getFileFilter()).getAlternativeExtension();
		if(alternativ != null){
		    if(!((returnFile.getName().toLowerCase().endsWith(extension)) ||
		       (returnFile.getName().toLowerCase().endsWith(alternativ))))
			returnFile = new File(returnFile.getPath()+extension);
		}
		else if(!(returnFile.getName().toLowerCase().endsWith(extension)))
		    returnFile = new File(returnFile.getPath()+extension);
	    }
	}
	//reset the fileFilters to null
	if(filterDefiner.length > 1)
	    setAcceptAllFileFilterUsed(false);
	for (int i = 0; i<filterDefiner.length;i++){
	    switch (filterDefiner[i]){
	    case JPG:
		removeChoosableFileFilter(jpgFilter);
		break;
	    case GIF:
		removeChoosableFileFilter(gifFilter);
		break;
	    case PNG:
		removeChoosableFileFilter(pngFilter);
		break;
	    case CDX:
		removeChoosableFileFilter(cdxFilter);
		break;		
	    }
	}
	previewPane.resetImages();
	return returnFile;
     }
}	
