package cdox.gui;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.util.*;
import javax.imageio.*;
import javax.swing.*;
/**
 * This class represents a cow in a lowered rectangle which works like a
 * trafficlight, if the thing you're doing is working it is red and the cow is sleeping, if
 * you're ready calculating something it's green... and awake. Easy eh.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version 25 april 2002
 * @see ImageDialog
 */
public class StatusDrawer extends JPanel{
    private Dimension size;
    private BufferedImage cdoxReady, cdoxWorking;
    private BufferedImage drawImage;
    private Window window;
    private AffineTransform scaled;
    private AffineTransformOp scaledTransform;
    private float scaleFactorX = 1f;
    private float scaleFactorY = 1f;
    private final float xPercentageOfy = 0.859375f;
    private HashMap cursorMap=null;
    private ArrayList windowComponents=null;

    /**
     *The constructor calls the other constructor with a height of 64 pixels, the
     *standard height of the cow.
     *@param wd the window in which this StatusDrawer will be used.
     */
    public StatusDrawer(Window wd){
	this(wd,64);
    }
    
    /**
     *This constructor loads the images of the cows, and calls resize to do some
     *calculating on the images. If an error in the loading happens, message dialog is
     *upped and the message is written in the errorlog.
     *@param wd the window in which this StatusDrawer will be used.
     *@param height the height of this component, the size will be adjusted.
     */
    public StatusDrawer(Window wd, int height){
	window = wd;
	try{
	    cdoxWorking = ImageIO.read(getClass().getResource("/cdox/images/cdoxworking.png"));
	    cdoxReady = ImageIO.read(getClass().getResource("/cdox/images/cdoxready.png"));
	}
	catch(Exception e){
	    CDoxFrame.handleError(new Throwable("Error in StatusDrawer",e), true);
	}
	resize(height);
	drawImage = cdoxReady;
	setBorder(BorderFactory.createLoweredBevelBorder());
    }


    /**
     *This method calculates the scale of the image. First the width is calculated which
     *is 0.859...*height (the original image = 55 x 64 check it out if you don't trust me)
     *An AffineTransform is made and its Op, which will be used to draw the images. Also
     *the minimum, maximum and preferred sizes are set here.
     */
    private void resize(int height){
	int width = Math.round(((float)height)*xPercentageOfy);
	if(height !=64){
	    scaleFactorX = ((float)width)/((float)55);
	    scaleFactorY = ((float)height)/((float)64);
	}
	scaled = AffineTransform.getScaleInstance(scaleFactorX,scaleFactorY);
	scaledTransform = new AffineTransformOp
	    (scaled,new RenderingHints(RenderingHints.KEY_COLOR_RENDERING,
				       RenderingHints.VALUE_COLOR_RENDER_SPEED));
	size = new Dimension(width+4, height+4);
	setMinimumSize(size);
	setPreferredSize(size);
	setMaximumSize(size);
    }

    /**
     *This method can be called if the Pane should be smaller, or larger. Take notice
     *though this is for setting the image height, the component height will be 4 pixels
     *higher (For drawing border reasons).
     *@param height the new height of the <B>image</B>.
     */
    public void setSize(int height){
	resize(height);
    }
	
    /**
     *Sets the status of this statusDrawer, true is ready false is not. It also sets the
     *cursor to waiting...That means all components in the CDoxFrame must be set to
     *waiting.
     *@param r true if calculating false if ready.
     *@see ImageDialog#update
     */
    public void setWorking(boolean r){
	if(!r){
	    drawImage = cdoxReady;
	    window.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	    if(windowComponents != null){
		if(cursorMap != null){
		    for(int i = 0; i<windowComponents.size();i++){
			((Component)windowComponents.get(i))
			    .setCursor((Cursor)cursorMap.get(windowComponents.get(i)));
		    }
		}
	    }
	}
	else{
	    drawImage = cdoxWorking;
	    window.setCursor(new Cursor(Cursor.WAIT_CURSOR));
	    if(windowComponents != null){
		if(cursorMap != null){
		    for(int i = 0; i<windowComponents.size();i++){
			((Component)windowComponents.get(i))
			    .setCursor(new Cursor(Cursor.WAIT_CURSOR));
		    }
		}
	    }
	}
	repaint();
    }

    /**
     * This method registers all Elements contained in the CDoxFrame, with this class in
     * an ArrayList. This method gets all original Cursors and stores them in a
     * hashmap. If the cow is set to be free all these original must be restored.
     * @param ac the ArrayList containing all CDoxFrame's components
     * @see #setWorking
     */
    public void registerComponents(ArrayList ac){
	windowComponents = ac;
	cursorMap = new HashMap();
	for(int i = 0; i<windowComponents.size();i++){
	    cursorMap.put(((Component)windowComponents.get(i)), ((Component)windowComponents.get(i)).getCursor());
	}
    }

    /**
     *Paints the images on this panel by using a drawImage with a BufferedImage and an
     *AffineTransformOp. Afterwards the borders are painted.
     *@param g the graphics Object to draw on.
     */
    public void paint(Graphics g){
	Graphics2D g2d =(Graphics2D)g;
	g2d.drawImage(drawImage,scaledTransform,2,2);
	paintBorder(g);
    }
}
		
    
