package cdox.gui;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * This Class arranges the layout of the toolbars, for the original JPanel can not handle a
 * GridBagLayout with a floating ToolBar. This Class overwrites the add method used (I
 * quess) internally in JToolBar to add Toolbars to a component. It took a little time to
 * find out which one they used, but I did :-). This class also handles the moving of the
 * TextToolbar which can either beneath the standard toolbar or to the right of it.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version 18 april 2002.
 * @see CDoxFrame#CDoxFrame
 */
public class ToolBarPane extends JPanel{
    private GridBagConstraints gb;
    private Point endPoint;
    private CDoxFrame cFrame;
    private MyComponentListener mcl;
    private boolean isAtTop = false;
    private ToolBarPane itsme = this;
    private int height=0;
    /**
     * Constructs a new JPanel with the specified layoutmanager. It also defines a
     * MyComponentListener (inner-class) which handles the dragging of the toolbar.
     * @param owner the owner of this JPanel.
     * @param lm the layoutManager to use.*GridBag*
     */
    public ToolBarPane(CDoxFrame owner, LayoutManager lm){
	super(lm);
	cFrame = owner;
	mcl = new MyComponentListener();
    }

    /**
     * This Method is called when a ToolBar is floating, and than closed, or moved by the
     * user. It adds the ToolBar to this container, with this GridBagConstraints
     * Object.
     * @param s The name of the ToolBar,
     * @param c the ToolBar.
     * @return the Component added.
     * @see #setConstraints
     * @see #setStandardToolBar
     */
    public Component add(String s, Component c){
	//The Toolbar can be set horizontal, if so put back vertical.(just for the beauty)
	if(((JToolBar)c).getOrientation() == JToolBar.VERTICAL){
	    JToolBar tBar = (JToolBar)c;
	    tBar.setOrientation(JToolBar.HORIZONTAL);
	    //and its three seperators.
	    ((JSeparator)tBar.getComponentAtIndex(0)).setOrientation(JToolBar.HORIZONTAL);
	    ((JSeparator)tBar.getComponentAtIndex(2)).setOrientation(JToolBar.HORIZONTAL);
	    ((JSeparator)tBar.getComponentAtIndex(5)).setOrientation(JToolBar.HORIZONTAL);
	}
	//clone the constraints used for the toolbar if it is beneath.
	GridBagConstraints gbc = (GridBagConstraints)gb.clone();
	gbc.weightx = 0;
	gbc.weighty=2000.0;
	gbc.anchor = gbc.NORTHWEST;	
	if(!isAtTop){
		/* if the TextToolbar is not at the top (beneath the standardToolbar)
		 * check the component beneath the mouse and the endpoint must be higher as the
		 * start point and the TextToolbar must fit (there is room between the right side
		 * of the window and default Toolbar.
		 */
		if((getComponentAt(endPoint)==null) && //
		   ((endPoint.y <0) && (endPoint.y >-33)) &&//
		   (((JToolBar)c).getWidth() + cFrame.getStandardToolBar().getWidth()) < getWidth()){
		    setStandardToolBar(gbc);//remove and add the defaultToolbar
		    gbc.gridx = 1;//for TextToolbar
		    gbc.weightx = 1.0;//a lot of weight in x
		    super.add(c, gbc);//add the TextToolbar
		    isAtTop = true;//and the toolbar is at the top.
		}
		else {//the TextToolbar stays down.
		    setStandardToolBar(gb);
		    super.add(c,gb);
		}
	    }
        else if (isAtTop){//the TextToolbar is at the top (to the right of the standardToolbar)
	    if((getComponentAt(endPoint)==null) &&
	       ((endPoint.y >16) && (endPoint.y <getHeight()))){
		setStandardToolBar(gb);//use beneath gridbag for DefaultToolbar
		super.add(c, gb);//add TextToolbar
		isAtTop = false;//and the TextToolbar is down.
	    }
	    else {//stays up. must be done, for this method is overwritten.
		setStandardToolBar(gbc);
		gbc.gridx = 1;
		gbc.weightx = 1.0;
		super.add(c,gbc);
	    }
	}
       redraw();
       return c;
    }

    /**
     * This method sets the standardToolBar with the specified Constraints.
     * @param c the new Constraints.
     */
    public void setStandardToolBar(GridBagConstraints c){
	JToolBar tBar = (JToolBar)getComponent(0);
	removeAll();
	super.add(tBar, c);
    }
	
    /**
     * This method first sets the standard toolbar with the new constraints, than removes
     * the component. This method is called every time the TextToolBar is dragged.
     * @param c the TextToolbar
     */
    public void remove(Component c){
	GridBagConstraints gbc = (GridBagConstraints)gb.clone();
	gbc.weightx = 2000.0;
	gbc.weighty=2000.0;
	gbc.anchor = gbc.NORTHWEST;	
	setStandardToolBar(gbc);
	super.remove(c);
	redraw();
    }

    /**
     * This method just redraws this panel, that way the cow is drawn to its writefull
     * height.
     */
    public void redraw(){
	if(height !=getPreferredSize().height){
	    cFrame.getStatusDrawer().setSize(getPreferredSize().height);
	    height=getPreferredSize().height;
	}
	cFrame.paintComponents(cFrame.getGraphics());
    }
    
    /**
     * This method sets the Constraints for the floating toolbar.
     * @param g The Constraints of the (Text)ToolBar.
     */
    public void setConstraints(GridBagConstraints g){
	gb = g;
    }

    /**
     * Returns the mousehandler for the TextToolBar.
     * @return the mousehandler for the TextToolbar.
     * @see CDoxFrame#CDoxFrame
     */
    public MyComponentListener getListener(){
	return mcl;
    }
    
    /**
     * An innerclass which handles the TextToolbar.
     */
    public class MyComponentListener extends MouseInputAdapter{
	/**
	 * Sets the point of the current mouseposition when the textToolbar is dragged.
	 * @param me the MouseEvent generated by dragging the textToolbar
	 */
	public void mouseDragged(MouseEvent me){
	    endPoint = me.getPoint();
	}
	/**
	 * Sets the point of the current mouseposition when the mouse-button is released.
	 * @param me the MouseEvent generated by releasing the mouse-button
	 */
	public void mouseReleased(MouseEvent me){
	    endPoint = me.getPoint();
	}
    }
}
