package cdox.util;

import java.io.*;
import java.util.*;
import javax.swing.*;

/**
 * This is a utility class that can be used for, yes, error handling. There are several
 * methods that take a throwable as argument. To use this class to write some exception
 * output into a file/stream, one must use one of the setErrorFile methods to initialize
 * the static fields. Please don't forget to #closeErrorLog the error log after using it
 * (it uses a buffered writer, so maybe there will not even be one char in it if you don't
 * close it).
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 22nd 2002
 *@see #closeErrorLog
 */
public class ErrorHandler{

    private static PrintWriter out=null;
    private static String location=null;
    private static Localizer lang=Localizer.getLocalizer(Locale.getDefault().getISO3Language());

    /**
     * If you don't set the language here, the system's default language is used. If it
     * does not exist, english is used.
     *@param language the new language as ISO-3-letter-language code.
     */
    public static void setLanguage( String language ){
	lang=Localizer.getLocalizer(language);
    }

    /**
     * Sets the output to the file that is specified in the String.
     *@param s the name of the file to write to.
     *@throws IOException if the specified path is not writeable.
     */
    public static void setErrorFile( String s ) throws IOException{
	setErrorFile(new File(s));
    }

    /**
     * Sets the output to the file.
     *@param f the file.
     *@throws IOException if the specified file is not writeable.
     */
    public static void setErrorFile( File f ) throws IOException{
	setErrorFile(new FileOutputStream(f));
	location=f.toString();
    }

    /**
     * Sets the output to the stream.
     *@param out the stream to write to.
     *@throws IOException if the specified stream is not valid.
     */
    public static void setErrorFile( OutputStream out ) throws IOException{
	ErrorHandler.out=new PrintWriter(new BufferedWriter(new OutputStreamWriter(out)));
    }

    /**
     * Writes the stack trace of e into the previously set output, if any. If the display
     * parameter is true, a message is displayed with the localized message of the
     * Throwable.
     *@param e the Throwable.
     *@param display if true, a message dialog is displayed indicating that an error has
     * occured. If the class has been initialized with a file or a string, the location of
     * the log file will also be displayed.
     */
    public static void log( Throwable e, boolean display ){
	if(display) showMessage(e);
	if(out!=null) e.printStackTrace(out);
    }

    /**
     * Shows a standard message dialog containing the localized message of the
     * Throwable. If the class has been initialized with a string or a file, the location
     * of the log is also told.
     *@param e the Throwable.
     */
    public static void showMessage( Throwable e ){
	if(lang!=null){
	    JOptionPane.showMessageDialog
		(null,e.getLocalizedMessage()+(((location==null)||(out==null))?"":
					       ("\n"+lang.get("loglocation",new String[]{location}))),lang.get("error"),
		 JOptionPane.ERROR_MESSAGE);
	}
    }

    /**
     * Logs the stack trace of the Throwable into the previously set output.
     *@param e the Throwable.
     */
    public static void log( Throwable e ){
	if(out!=null) e.printStackTrace(out);
    }

    /**
     * Closes the error log file. This one should be called, if the logging mechanism is used!
     */
    public static void closeErrorLog(){
	if(out!=null) out.close();
    }

}
