package cdox.util;

import java.util.*;
import java.io.*;

/**
 *<p>
 * This is a wrapper class that handles several localized language files. To use this, you must
 * have some language files somewhere. If you have a package com.microsoft.stupid you could
 * have a dir called lang there with a language file `deu'. Then you would call new
 * Localizer() or setLang() with (Locale.GERMAN,"/com/microsoft/stupid/lang/") to initialize
 * this class.
 *</p>
 *<p>
 * The language files must be named according to ISO 639-2/T (three letter language code),
 * english would be "eng", german "deu" etc.
 *</p>
 *<p>
 * The language files can also be used to provide you with keystrokes. If
 * provideKeystrokes is set to true, every string that is returned will be stripped from
 * &amp; chars. If set to false (which is the default), the &amp; chars are also returned.
 *</p>
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 22nd 2002
 */

public class Localizer{

    private PropertyResourceBundle lang=null;
    private boolean keystrokes=false;
    private String language=null;
    private String langPackage=null;

    /**
     * Constructs a new Localizer. Throws an IOException if the language file had not been found.
     *@param loc the locale which language should be used
     *@param path the path to the new language file
     *@throws IOException if no language file for loc had been found
     */
    public Localizer( Locale loc, String path ) throws IOException{
	if(!setLang(loc,path)) throw new IOException("Language file not found for "+
						     loc.getDisplayLanguage(Locale.ENGLISH)+".");
    }

    /**
     * Creates a new <code>Localizer</code> instance.
     *
     * @param lng a <code>String</code> value indicating the language.
     * @param path a <code>String</code> value indicating the package path.
     * @exception IOException if an error occurs.
     */
    public Localizer( String lng, String path ) throws IOException{
	langPackage=path;
	if(!setLang(path+lng)) throw new IOException("Language file not found.");
    }

    /**
     * Sets the new language for this Localizer. Returns whether the new language exists or not.
     *@param loc the locale which language should be used
     *@param path the path to the new language file
     *@return true if the new language had set, otherwise false.
     */
    public boolean setLang( Locale loc, String path ){
	language=loc.getISO3Language();
	langPackage=path;
	return setLang(path+loc.getISO3Language());
    }

    /**
     * Sets the new language for this Localizer. Returns whether the new language exists or not.
     *@param path the path to the new language file, ending with the iso3 language code.
     *@return true if the new language had set, otherwise false.
     */
    public boolean setLang( String path ){
	language=path.substring(path.lastIndexOf('/')+1);
	try{
	    lang=new PropertyResourceBundle(getClass().getResourceAsStream
					    (path));
	}catch(IOException e){
	    return false;
	}catch(NullPointerException npe){
	    return false;
	}
	return lang!=null;
    }

    /**
     * Returns the ISO3 language code for this localizer.
     *@return the code.
     */
    public String getLanguageCode(){
	return language;
    }

    /**
     * Returns the available languages. There must exist a properties file named
     * <code>availablelanguages</code> in the languages package where all other language
     * files are for this to work. If no such file exists, an empty properties object will
     * be returned.
     *@return a properties object with the ISO3-Languages as values and the display
     * languages as keys.
     */
    public Properties getAvailableLanguages(){
	Properties props=new Properties();
	try{
	    props.load(getClass().getResourceAsStream
		       (langPackage+"availablelanguages"));
	}catch(IOException ioe){
	    ErrorHandler.log(ioe,true);
	}
	return props;
    }

    /**
     * Sets whether the Localizer should provide you with keystrokes and strip the &amp;
     * characters or not.
     *@param yes true if you want the keystrokes.
     */
    public void provideKeystrokes( boolean yes ){
	keystrokes=yes;
    }

    /**
     * Returns whether the Localizer provides you with keystrokes and strips the &amp;
     * character or not.
     *@return true, if he does.
     */
    public boolean providesKeystrokes(){
	return keystrokes;
    }

    /**
     * Returns the key stroke for this object or ' ', if none has been specified.
     *@param key the key.
     *@return the char.
     */
    public char getStroke( String key ){
	if(!keystrokes) return ' ';
	if(lang.getString(key).indexOf("&")==-1) return ' ';
	return lang.getString(key).toUpperCase().charAt(lang.getString(key).indexOf("&")+1);
    }

    /**
     * Returns the localized String mapping the key. Please note that possible &amp;
     * charachters will be removed from the String.
     *@param key the key
     *@return the String
     */
    public String get( String key ){
	return stripAmp(lang.getString(key));
    }

    /**
     * Returns the localized String mapping the key. The second parameter contains the
     * Strings that should be used to replace the parameters. Such parameters occur in the
     * language file as %1% %2% %3% etc. This method will throw an exception if anything
     * goes wrong (I will make proper error handling later). Please note that possible
     * &amp; characters are removed from the String.
     *@param key the key
     *@param param the replacements.
     *@return the String
     */
    public String get( String key, String[] param ){
	StringBuffer sb=new StringBuffer(stripAmp(lang.getString(key)));
	for(int i=0;i<param.length;i++){
	    String h="%"+(i+1)+"%";
	    sb.replace(sb.toString().indexOf(h),
		       sb.toString().indexOf(h)+h.length(),
		       param[i]);
	}
	return sb.toString();
    }

    // strips any "&" from the string
    private String stripAmp( String s ){
	if(!keystrokes) return s;
	StringBuffer sb=new StringBuffer(s);
	while(sb.toString().indexOf("&")!=-1) sb.deleteCharAt(sb.toString().indexOf("&"));
	return sb.toString();
    }

    /**
     * This method just returns a Localizer for the utilities packages. One should not use
     * this one outside the util package. Errors will be logged into the ErrorHandler log,
     * if it exists.
     *@param language the language for the localizer.
     *@return the Localizer or null, if no suitable language file was found.
     */
    public static Localizer getLocalizer( String language ){
	try{
	    return new Localizer(language,"/cdox/util/lang/");
	}catch(IOException ioe){
	    ErrorHandler.log(ioe,true);
	}
	return null;
    }

}
