package cdox.util;

import java.awt.image.*;
import java.io.*;
import java.util.*;
import java.util.zip.*;
import javax.imageio.*;
import javax.imageio.stream.*;

/**
 * This is a utility class that takes care of some temporarily files. One gets a unique
 * key (the temp file name) to get an input stream out of the get method ("got" it?)(yes,
 * probably a bad one).
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 22nd 2002
 */
public class TempFiles{

    private static HashMap tempFiles=new HashMap();

    /**
     * This method adds a temp file that has already been created. Please note that f is
     * not removed automatically, one must invoke the deleteOnExit() method before adding
     * it here.
     *@param key the temp file key.
     *@param f the temp file.
     */
    public static void add( String key, File f ){
	tempFiles.put(key,f);
    }

    /**
     * This method adds a temp file.
     *@param f the file to read the temp file from.
     *@return a unique string identifying the resource.
     *@throws IOException if the temporary file could not be created or the original file
     * could not be read.
     */
    public static String add( File f ) throws IOException{
	// creates a new tmp file and copies the file into it
	File tmp=File.createTempFile("cdox.",null);
	tmp.deleteOnExit();
	copy(new FileInputStream(f),tmp);
	return tmp.toString();
    }

    /**
     * This method adds a temp file. The ZipFile will not be closed!
     *@param f the file to read the temp file from.
     *@param e the zip entry to use.
     *@return a unique string identifying the resource.
     *@throws IOException if the temporary file could not be created or the original file
     * could not be read.
     */
    public static String add( ZipFile f, ZipEntry e ) throws IOException{
	// creates a new tmp file and copies the zip entry into it
	File tmp=File.createTempFile("cdox.",null);
	tmp.deleteOnExit();
	copy(f.getInputStream(e),tmp);
	return tmp.toString();
    }

    /**
     * This method adds an image as temp file. This ones can then also be read with the
     * getImage method.
     *@param image the image to temporary store.
     *@return a unique string identifying the resource.
     *@throws IOException if the temporary file could not be created.
     */
    public static String add( BufferedImage image ) throws IOException{
	// creates a new tmp file and copies the zip entry into it
	File tmp=File.createTempFile("cdox.",null);
	tmp.deleteOnExit();
	tempFiles.put(tmp.toString(),tmp); // the temp file is added here to the map
	ImageIO.write(image,"png",tmp);
	return tmp.toString();
    }

    // copies from the input stream into the output stream
    private static void copy( InputStream ins, File outf ) throws IOException{
	tempFiles.put(outf.toString(),outf); // the temp file is added here to the map
	BufferedInputStream in=new BufferedInputStream(ins); // just copy the bytes
	BufferedOutputStream out=new BufferedOutputStream(new FileOutputStream(outf));
	byte[] buf = new byte[4096];
	int len;
	while ((len = in.read(buf)) > 0) {
	    out.write(buf, 0, len);
	}
	out.close();
	in.close();
    }

    /**
     * Returns the input stream for the temp file associated with the specified name.
     *@param s the key for the temp file.
     *@return the input stream or null, if something went wrong.
     */
    public static BufferedInputStream get( String s ){
	try{
	    return new BufferedInputStream
		(new FileInputStream((File)tempFiles.get(s)));
	}catch(FileNotFoundException fnfe){ // huh, did we create the file or not?
	    ErrorHandler.log(fnfe,true);
	}
	return null;
    }

    /**
     * Returns a buffered image that has previously been stored in a temp file.
     *@param s the key for the temp file.
     *@return the image or null, if something went wrong.
     */
    public static BufferedImage getImage( String s ){
	try{
	    return loadImage(get(s));
	}catch(IOException ioe){
	    ErrorHandler.log(ioe,true);
	}
	return null;
    }

    /**
     * Reads an image out of the stream. This one creates a BufferedImage of TYPE_INT_ARGB.
     * @param in where to read the image.
     * @return the image.
     * @exception IOException if an error occurs.
     */
    public static BufferedImage loadImage( InputStream in ) throws IOException{
	BufferedImage img=ImageIO.read(in);
	ColorConvertOp op=new ColorConvertOp(null);
	// hopefully no memory trouble with that one.
	if(img==null) throw new IOException("Format not supported.");
	return op.filter(img,new BufferedImage(img.getWidth(),img.getHeight(),BufferedImage.TYPE_INT_ARGB));
    }

}
