package cdox.util.conf;

import cdox.util.*;
import java.util.*;
import java.util.prefs.*;
import javax.swing.*;
import org.w3c.dom.*;

/**
 * This class is the abstract superclass for various options.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 22nd 2002
 */
public abstract class AbstractOption{

    /** The preferences node for this option. */
    protected Preferences prefs;
    /** The preferences key for this option. */
    protected String key;
    /** The DOM node where this option is described. */
    protected Element node;

    /**
     * Constructs new one. The DOM node must be the element node of this option, for
     * example the "keystroke" node (for a keystroke).
     *@param cls a string that is used to determine the preferences node where the option
     * will be stored.
     *@param key the preferences key.
     *@param n the DOM node where this option is described.
     */
    protected AbstractOption( String cls, String key, Node n ){
	try{
	    prefs=Preferences.userNodeForPackage(Class.forName(cls));
	}catch(ClassNotFoundException cnfe){
	    ErrorHandler.log(cnfe,true);
	}
	this.key=key;
	node=(Element)n;
    }

    /**
     * Returns a JLabel which describes the option. This method should be invoked only
     * once.
     *@return a JLabel.
     */
    public abstract JLabel getText();

    /**
     * Returns a component which is used to alter this option. This method should be
     * invoked only once.
     * @return a <code>JComponent</code> value.
     */
    public abstract JComponent getComponent();

    /**
     * Causes the option to write its changes to its preferences node. The preferences'
     * flush() method is <i>NOT</i> invoked by this method.
     *@return true, if the application must be restarted in order to get the changes into
     * effect.
     */
    public abstract boolean commitChanges();

    /**
     * Causes the preferences to re-read its default value from the DOM document and thus
     * resetting its value in the component.
     */
    public abstract void setToDefault();

    /**
     * Resets the values to the original ones, either from the preferences, or, if they
     * don't exist, to the ones in the XML file.
     */
    public abstract void reset();

    /**
     * This method might be overridden by subclasses that need to provide its values as
     * objects. The mapping should use the key value used in the xml file as key value.
     *@param map the map where to store the mappings.
     */
    public void setMapping( HashMap map ){
    }

    /**
     * Returns the #text node value of the matching element (if any). Uses the localizer
     * to determine the element which matches the language of the localizer (by comparing
     * the language to the xml:lang attribute).
     *@param parent the node, which must be the parent of matching elements.
     *@param element the name of the element that is searched for.
     *@param lang the localizer whose language should be searched for.
     *@return the text or null, if no language/element pair matches the language.
     */
    protected static String getTextByLanguage( Node parent, String element,
					       Localizer lang ){
	Document doc=parent.getOwnerDocument();
	NodeList nl=doc.getElementsByTagName(element);
	for(int i=0;i<nl.getLength();i++){
	    if(nl.item(i).getParentNode()!=parent) continue;
	    String v=nl.item(i).getAttributes().getNamedItem("xml:lang").getNodeValue();
	    if(lang.getLanguageCode().startsWith(v)||
	       v.startsWith(lang.getLanguageCode()))
		return nl.item(i).getFirstChild().getNodeValue();
	}
	return null;
    }

}
