package cdox.util.conf;

import cdox.util.*;
import java.awt.event.*;
import java.lang.reflect.*;
import java.util.*;
import javax.swing.*;
import org.w3c.dom.*;

/**
 * This class represents a KeyStroke option. Below is an example of an appropriate
 * section in the XML file:
 *<pre>
 * &lt;keystroke package="cdox.gui.CDoxFrame" key="about"&gt;
 *   &lt;default key="VK_A" modifier="CTRL_MASK+SHIFT_MASK"/&gt;
 *   &lt;hint xml:lang="de"&gt;Infos ber CDox&lt;/hint&gt;
 *   &lt;hint xml:lang="en"&gt;Show about box&lt;/hint&gt;
 * &lt;/keystroke&gt;
 *</pre>
 * You can use several modifier keys in that file, but they must be separated by "+"
 * chars. You find the constants you can use in the class java.awt.event.KeyEvent, for
 * modifiers you must use the *MASK constants from InputEvent.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 22nd 2002
 */
public class KeyStrokeOption extends AbstractOption{

    private Localizer lang;
    private JKeyStrokeField field;
    // the next one is to check double key strokes
    private static HashSet keystrokes=new HashSet();

    /**
     * Constructs one.
     *@param n the DOM node where this option is described.
     *@param lang a localizer.
     */
    public KeyStrokeOption( Node n, Localizer lang ){
	super(n.getAttributes().getNamedItem("package").getNodeValue(),
	      n.getAttributes().getNamedItem("key").getNodeValue(),n);
	this.lang=lang;
	keystrokes.add(this);
    }

    public JLabel getText(){
	return new JLabel(getTextByLanguage(node,"hint",lang)+":");
    }

    public JComponent getComponent(){
	field=new JKeyStrokeField(0,0);
	reset();
	return field;
    }

    public boolean commitChanges(){
	int[] keys=field.getValue();
	reset();
	boolean restart=true;
	if((field.getValue()[0]==keys[0])&&(field.getValue()[1]==keys[1]))
	    restart=false;
	field.setValue(keys[0],keys[1]);
	prefs.put(key,field.getValue()[0]+"+"+field.getValue()[1]);
	return restart;
    }

    public void setToDefault(){
	try{
	    NodeList nl=node.getChildNodes();
	    Node def=null;
	    for(int i=0;i<nl.getLength();i++)
		if(nl.item(i).getNodeName().equals("default")) def=nl.item(i);
	    NamedNodeMap nnm=def.getAttributes();
	    String defkey=nnm.getNamedItem("key").getNodeValue();
	    String defmod=nnm.getNamedItem("modifier").getNodeValue();
	    Field f=KeyEvent.class.getDeclaredField(defkey);
	    int keyval=f.getInt(f);
	    StringTokenizer t=new StringTokenizer(defmod,"+");
	    int modval=0;
	    while(t.hasMoreTokens()){
		f=InputEvent.class.getDeclaredField(t.nextToken());
		modval+=f.getInt(f);
	    }
	    field.setValue(keyval,modval);
	}catch(NoSuchFieldException nsfe){
	    ErrorHandler.log(nsfe,true);
	}catch(IllegalAccessException iae){
	    ErrorHandler.log(iae,true);
	}
    }

    public void reset(){
	String s=prefs.get(key,null);
	if(s==null){
	    setToDefault();
	    field.setColumns(12);
	    return;
	}
	StringTokenizer t=new StringTokenizer(s,"+");
	field.setValue(Integer.parseInt(t.nextToken()),
		       Integer.parseInt(t.nextToken()));
	field.setColumns(12);
    }

    /**
     * Maps the key stroke for this option to its key. Please note that this will set a
     * key stroke corresponding to the values in the component, no matter if the changes
     * have been already committed or not. This method is useful to provide an easy way to
     * map keystrokes to actions.
     *@param map the hash map where the value will be put.
     *@see AbstractOption#setMapping
     */
    public void setMapping( HashMap map ){
	map.put(key,getStroke());
    }

    // just returns the stroke.
    private KeyStroke getStroke(){
	return KeyStroke.getKeyStroke(field.getValue()[0],field.getValue()[1]);
    }

    /**
     * This one returns the descriptive hint for this keystroke as String.
     *@return the text as String.
     */
    protected String getHint(){
	return getTextByLanguage(node,"hint",lang);
    }

    /**
     * This method checks all changed keystrokes for validity. It returns a KeyStrokeOption[]
     * with two KeyStrokeOptions that have the same key stroke, or null, if everything is
     * ok.
     *@return a KeyStrokeOption[] with two double key stroke options or null, if
     * everything is ok.
     */
    public static KeyStrokeOption[] checkValidity(){
	HashSet set=new HashSet();
	HashSet set2=new HashSet();
	set.addAll(keystrokes);
	set2.addAll(keystrokes);
	Iterator i=set.iterator();
	while(i.hasNext()){
	    KeyStrokeOption o=(KeyStrokeOption)i.next();
	    set2.remove(o);
	    Iterator j=set2.iterator();
	    while(j.hasNext()){
		KeyStrokeOption oh=(KeyStrokeOption)j.next();
		if(o.getStroke().equals(oh.getStroke())) return new KeyStrokeOption[]{o,oh};
	    }
	}
	return null;
    }

    /**
     * Resets the validity information to zero. Currently just <code>clear()</code>s an
     * internal hash set. This should be done every time the configuration dialog is
     * initialized/created. Well, actually, it <i>is</i> done in its constructor...
     */
    public static void resetValidityCheck(){
	keystrokes.clear();
    }

    public String toString(){
	return getHint();
    }

}
