package cdox.util.doc;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.text.html.*;
import java.io.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * A browser that is used to browse the help file(s). The browser caches the topics
 * automatically. The cache is cleared, if the language is set.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 22nd 2002
 */
public class Browser extends JDialog implements HyperlinkListener{

    private String topic;
    private Documentation doc;
    private JEditorPane text,tree;
    private AbstractAction backAction,fwdAction,upAction;
    private ArrayList history=new ArrayList();
    private HashMap ancestors,categories,cache=new HashMap();
    private int historyIndex=0;
    private String language=Locale.getDefault().getLanguage();

    /**
     * Constructs new one.
     * @param owner a <code>JFrame</code> value indicating the owner.
     * @param topic the default topic.
     * @param doc the Documentation.
     */
    public Browser( JFrame owner, String topic, Documentation doc ){
	super (owner,true);
	this.topic=topic;
	this.doc=doc;
	HashMap[] maps=doc.getCategories(language);
	categories=maps[1];
	ancestors=maps[0];
	components();
    }

    /**
     * Constructs new one.
     * @param owner a <code>JFrame</code> value indicating the owner.
     * @param doc the Documentation.
     */
    public Browser( JFrame owner, Documentation doc ){
	this(owner, doc.getDefaultTopicName(),doc);
    }

    /**
     * This method clears the history and sets the current topic as the only item of it.
     */
    public void clearHistory(){
	history.clear();
	historyIndex=0;
	history.add(topic);
    }

    /**
     * Sets the language and clears the cache.
     *@param lang the language.
     */
    public void setLanguage( String lang ){
	try{
	    language=lang.substring(0,2);
	}catch(ArrayIndexOutOfBoundsException aioobe){
	    language=lang;
	}
	HashMap[] maps=doc.getCategories(language);
	cache.clear();
	categories=maps[1];
	ancestors=maps[0];
    }

    /**
     * Sets the new topic of this browser.
     *@param topic the new topic.
     */
    public void setTopic( String topic ){
	this.topic=topic;
	if(cache.get(topic)==null)
	    cache.put(topic,doc.getTopic(topic,language));
	text.setText(cache.get(topic).toString());
	if(ancestors.get(topic)==null) // in that case we are top level
	    tree.setText(doc.getTopicsAsString(language,null));
	else // further checking needed:
	    if(categories.get(topic).toString().equals("true")) // if true, we are in a category,
		tree.setText(doc.getTopicsAsString(language,topic)); // so use the
								     // subtopics of the
								     // requested one.
	// otherwise: take the subtopics of the ancestor (which results in a list with the
	// displayed topic included).
	    else tree.setText(doc.getTopicsAsString(language,(String)ancestors.get(topic)));
	tree.setCaretPosition(0);
	text.setCaretPosition(0);
    }

    private void components(){
	JToolBar bar=new JToolBar();
	bar.setFloatable(false);
	history.add(topic);
	bar.add(backAction=new AbstractAction
		("back",new ImageIcon(getClass().getResource("/cdox/images/back.png"))){
		public void actionPerformed( ActionEvent evt ){
		    if(historyIndex>0){
			setTopic(history.get(historyIndex-1).toString());
			historyIndex--;
		    }
		}
	    });
	bar.add(upAction=new AbstractAction
		("up",new ImageIcon(getClass().getResource("/cdox/images/up.png"))){
		public void actionPerformed( ActionEvent evt ){
		    if(ancestors.get(topic)==null) return; // top level is "" which has
							    // no category
		    setTopic(ancestors.get(topic).toString());
		    while((historyIndex+1)<history.size())
			history.remove(history.size()-1);
		    history.add(topic);
		    historyIndex++;
		}
	    });
	bar.add(fwdAction=new AbstractAction
		("fwd",new ImageIcon(getClass().getResource("/cdox/images/forward.png"))){
		public void actionPerformed( ActionEvent evt ){
		    if(historyIndex<(history.size()-1)){
			setTopic(history.get(historyIndex+1).toString());
			historyIndex++;
		    }
		}
	    });
	getContentPane().add(bar,BorderLayout.NORTH);
	setTitle(doc.getTitle(language));
	text=new JEditorPane();
	text.setContentType("text/html");
	text.getDocument().putProperty("IgnoreCharsetDirective",Boolean.TRUE); // Gotcha...
	if(cache.get(topic)==null) cache.put(topic,doc.getTopic(topic,language));
	text.setText(cache.get(topic).toString());
	text.setEditable(false);
	text.setCaretPosition(0);
	text.addHyperlinkListener(this);

	tree=doc.getTopics(language);
	tree.setCaretPosition(0);
 	tree.addHyperlinkListener(this);

	JScrollPane p1=new JScrollPane(tree);
	JScrollPane p2=new JScrollPane(text);
	JSplitPane panel=new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,p1,p2);
	panel.setDividerSize(5);
	panel.setResizeWeight(0.40);
	panel.setPreferredSize(new Dimension(400,300));

	getContentPane().add(panel);
	center();
    }
    
    public void hyperlinkUpdate( HyperlinkEvent e ){
	if(e.getEventType()==HyperlinkEvent.EventType.ACTIVATED){
	    setTopic(e.getDescription());
	    while((historyIndex+1)<history.size())
		history.remove(history.size()-1);
	    history.add(e.getDescription());
	    historyIndex++;
	}
    }

    private void center(){
	pack();
        Dimension ss = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension fs = getSize();
        setLocation((ss.width-fs.width)/2,(ss.height-fs.height)/2);
    }

}
