package cdox.util.doc;

import javax.swing.*;
import javax.swing.tree.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.dom.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

/**
 *<p>
 * This class grants easy access to an application's documentation. One has only to write
 * the documentation once in XML format (using the Schema
 * <a href="http://sammael.tabu.stw-bonn.de/schema/documentation.xsd">here</a>) and can
 * then use this class to display a help browser or a specific help text. This class is
 * also a command line utility to generate HTML out of the documentation.
 *</p>
 *<p>
 * You can also use this class to display several documentations in one help browser. For
 * ease of use, one can specify a class to be Documented (using the interface) and provide
 * several of this classes instead of the appropriate Documentations.
 *</p>
 *<p>
 * Depending on the amount of documentation one can use either one Documentation for the
 * application (and one class implements Documented) or several classes to be
 * Documented. If you use the methods that require several Documented classes as argument,
 * they are displayed as top-level documentation nodes.
 *</p>
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 22nd 2002
 */
public class Documentation{

    private Document doc;
    private TransformerFactory tf=TransformerFactory.newInstance();
    private Browser browser;

    /**
     * Constructs one using the given stream.
     * @param owner a <code>JFrame</code> value indicating a parent frame.
     * @param in the stream.
     * @exception IOException if an error occurs.
     */
    public Documentation(JFrame owner, InputStream in ) throws IOException{
	try{
	    DocumentBuilderFactory dbf=DocumentBuilderFactory.newInstance();
	    dbf.setNamespaceAware(true);
	    DocumentBuilder db=dbf.newDocumentBuilder();
	    setDocument(owner, db.parse(new InputSource(in)));
	}catch(ParserConfigurationException pce){
	    cdox.util.ErrorHandler.log(pce,true);
	}catch(SAXException saxe){
	    cdox.util.ErrorHandler.log(saxe,true);
	}
    }

    /**
     * Constructs one using the given file.
     * @param owner a <code>JFrame</code> value indicating a parent frame.
     * @param location the path to the file.
     * @exception IOException if an error occurs.
     */
    public Documentation(JFrame owner, String location ) throws IOException{
	this(owner, new FileInputStream(location));
    }

    /**
     * Constructs one using the given URL.
     * @param owner a <code>JFrame</code> value indicating a parent frame.
     *@param location the URL.
     * @exception IOException if an error occurs.
     */
    public Documentation(JFrame owner, URL location ) throws IOException{
	this(owner, location.openStream());
    }

    /**
     * Constructs one using the given file.
     * @param owner a <code>JFrame</code> value indicating a parent frame.
     * @param location the file.
     * @exception IOException if an error occurs.
     */
    public Documentation( JFrame owner, File location ) throws IOException{
	this(owner,new FileInputStream(location));
    }

    private void setDocument( JFrame owner, Document doc ){
	this.doc=doc;
	browser=new Browser(owner, this);
    }

    /**
     * Returns the DOM document for this documentation. Modify on your own risk.
     *@return the document.
     */
    public Document getDocument(){
	return doc;
    }

    /**
     * Displays the requested documentation in swing. This generates and displays a JFrame
     * containing the documentation regarding the topic.
     * @param id the topic to be displayed.
     * @param language the desired language.
     */
    public void displayDocumentationFor( String id, String language ){
	browser.setLanguage(language);
	browser.setTopic(id);
	browser.clearHistory();
	browser.show();
    }

    /**
     * Displays all documentation in swing.
     * @param language the desired language.
     */
    public void displayDocumentation( String language ){
	browser.setLanguage(language);
	browser.setTopic(getDefaultTopicName());
	browser.clearHistory();
	browser.show();
    }

    /**
     * Returns the title of this documentation.
     *@param language the desired language.
     *@return the title.
     */
    public String getTitle( String language ){
	NodeList nl=doc.getElementsByTagName("language");
	for(int i=0;i<nl.getLength();i++){
	    if(nl.item(i).getAttributes().getNamedItemNS
	       ("http://www.w3.org/XML/1998/namespace","lang").
	       getNodeValue().equals(language))
		return nl.item(i).getAttributes().getNamedItem("title").getNodeValue();
	}
	return null;
    }

    /**
     * Returns the requested topic's subtopics as HTML in a string. This will also return
     * a HTML document, if there exist no subtopics.
     *@param language the desired language.
     *@param topic the topic.
     *@return the HTML.
     */
    public String getTopicsAsString( String language, String topic ){
	StringWriter sw=new StringWriter();
	try{
	    Transformer t=tf.newTransformer
		(new StreamSource(ClassLoader.getSystemResourceAsStream
				  ("cdox/util/doc/xsl/32index.xsl")));
	    t.setParameter("language",language);
	    if(topic!=null) t.setParameter("father",topic);
	    t.transform(new DOMSource(doc),new StreamResult(sw));
	    sw.close();
	}catch(TransformerConfigurationException tce){
	    cdox.util.ErrorHandler.log(tce,true);
	}catch(TransformerException te){
	    cdox.util.ErrorHandler.log(te,true);
	}catch(IOException ioe){
	    cdox.util.ErrorHandler.log(ioe,true);
	}
	return sw.toString();
    }

    /**
     * Returns a JEditorPane with all the topics in the root of the documentation.
     *@param language the desired language.
     *@return the pane.
     */
    public JEditorPane getTopics( String language ){
	JEditorPane text=new JEditorPane();
	text.setContentType("text/html");
	text.setText(getTopicsAsString(language,null));
	text.setEditable(false);
	text.setCaretPosition(0);
	return text;
    }

    /**
     * Returns the requested topic as HTML in the String.
     *@param id the topic.
     *@param language the language.
     *@return the String.
     */
    public String getTopic( String id, String language ){
	if((id==null)||(id.equals(""))) id=getDefaultTopicName();
	StringWriter sw=new StringWriter();
	try{
	    Transformer t=tf.newTransformer
		(new StreamSource(ClassLoader.getSystemResourceAsStream
				  ("cdox/util/doc/xsl/32topic.xsl")));
	    t.setParameter("topic",id);
	    t.setParameter("language",language);
	    t.transform(new DOMSource(doc),new StreamResult(sw));
	    sw.close();
	}catch(TransformerConfigurationException tce){
	    cdox.util.ErrorHandler.log(tce,true);
	}catch(TransformerException te){
	    cdox.util.ErrorHandler.log(te,true);
	}catch(IOException ioe){
	    cdox.util.ErrorHandler.log(ioe,true);
	}
	return sw.toString();
    }

    /**
     * Returns the default topic id of this documentation.
     *@return the topic id.
     */
    public String getDefaultTopicName(){
	return doc.getElementsByTagNameNS("http://sammael.tabu.stw-bonn.de/schema/documentation","documentation").
	    item(0).getAttributes().getNamedItem("default").getNodeValue();
    }

    /**
     * Returns category related information in this documentation in two HashMaps. The
     * first map maps the topic ids to its direct ancestor, or the empty string, if none
     * exists. The second map stores "true", if the topic is a category (has subtopics)
     * and false, otherwise.
     *@param language the language from which the topics are desired.
     *@return the HashMaps with the mappings.
     */
    public HashMap[] getCategories( String language ){
	HashMap map=new HashMap();
	HashMap catMap=new HashMap();
	try{ // transform it via piped streams
	    final Transformer t=tf.newTransformer
		(new StreamSource(ClassLoader.getSystemResourceAsStream
				  ("cdox/util/doc/xsl/categories.xsl")));
	    t.setParameter("language",language);
	    PipedOutputStream po=new PipedOutputStream();
	    PipedInputStream pi=new PipedInputStream(po);
	    final PrintStream out=new PrintStream(po);
	    new Thread(){ // at least it works
		public void run(){
		    try{
			t.transform(new DOMSource(doc),new StreamResult(out));
			out.close();
		    }catch(TransformerException te){
			cdox.util.ErrorHandler.log(te,true);
		    }
		}
	    }.start();
	    BufferedReader in=new BufferedReader(new InputStreamReader(pi));
	    String s=in.readLine();
	    while(s!=null){ // something important I didn't know before: one should not
			    // use ready(), it MUST return null if there is EOF
		s=s.trim();
		if(!s.equals("")){
		    // see the stylesheet about the format that is parsed here
		    // (categories.xsl)
		    int index=s.indexOf(" ");
		    int lindex=s.lastIndexOf(' ');
		    if(index==lindex){
			map.put(s.substring(0,lindex),"");
			catMap.put(s.substring(0,lindex),
				   ""+(Integer.parseInt(s.substring(lindex+1))!=0));
		    }else{
			map.put(s.substring(index+1,lindex),s.substring(0,index));
			catMap.put(s.substring(index+1,lindex),
				   ""+(Integer.parseInt(s.substring(lindex+1))!=0));
		    }
		}
		s=in.readLine();
	    }
 	    in.close();
	}catch(IOException ioe){
	    cdox.util.ErrorHandler.log(ioe,true);
	}catch(TransformerConfigurationException tce){
	    cdox.util.ErrorHandler.log(tce,true);
	}
	return new HashMap[]{map,catMap};
    }

    /**
     * Sets a background image by URL. This image will then be the background image in all
     * displayed topics.
     *@param url the url. Please note that you can also use jar URLs as defined in
     * java.net.JarURLConnection.
     */
    public void setBackgroundURL(URL url){
	Node n = doc.getElementsByTagName("documentation").item(0);
	Attr at = doc.createAttribute("backgroundimage");
	at.setValue(url.toString());
	n.getAttributes().setNamedItem(at);
    }
    
}
