package cdox.util.image;

import java.util.Observer;
import java.awt.image.*;
import java.awt.*;
/**
 *This class highers or loweres the contrast of a BufferedImage. It uses a lookupTable to
 *do so. To use this class you can call one of its two constructors. If you've designed
 *your own Jslider classes you can use the constructor which takes a float-value. You than
 *can use the setValue(float val) method to set the value to work with. And than call the
 *filter method to filter the BufferedImage.If value == 0f the original (source-image)
 *will be returned.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 22nd 2002
 * @see CorrectorSlider
 */
public class Contrast extends CorrectorSlider implements ImageCorrector{
     /**
      *This constructor creates a new Contrast Object, with a JSlider in a JPanel with a
      *titledBorder set with the name paneTitle. The values of the JSlider are standardly
      *min=-100, max = 100, initValue=0, spacings=10, textatSpacing=25,
      *graptogrid=false. The Observer is called when the JSlider is moved. This Observer
      *could be a preview pane or anything else. 
      *@param obs The Observer to handle the BufferedImage.
      *@param paneTitle the title of the titledBorder of the JPane surrounding the JSlider.
      *@see CorrectorSlider#CorrectorSlider(Observer, String, int, int, int, int, int, boolean).
      */
    public Contrast(Observer obs, String paneTitle){
	super(obs, paneTitle, -100,100,0,10,25, false);
	imageCorrector = this;
    }
    /**
     *This constructor creates a new Contrast Object with its initial-Value set to val. It
     *has no JSlider and the value of this class (to work with) should be set by calling
     *setValue(float val).
     *@param val the initial float value.
     *@see CorrectorSlider#CorrectorSlider(float)
     *@see CorrectorSlider#setValue
     */
    public Contrast(float val){
	super(val);
    }
    /**
     * <p>The actual filtering is done in this method defined in ImageCorrector. It uses a
     *ShortLookupTable to make the contrast higher or lower for all the four chanels (Red,
     *Green, Blue, Alpha). First the short-array is filled with the values, if the
     *colornumber = i(0-255)>127, a hundreth*i-127*value <I>(set by the JSlider or
     *setValue(float val))</I> is added to the colornumber, causing it to be slightly
     *higher / lower as the original color in the middle (80 - 170) and a lot higher/lower
     *for the low/high colors (black (i=0), white(i=255)). After that this short-array is
     *copied in a 2-dimensional-short-array, which represent the color-chanels. With
     *BufferedImageOp a new LookupOp is done, which will be used to filter the source
     *BufferedImage.</p>    
     *<p>To make sure the source-BufferedImage can be "contrasted" the image is redrawn on
     *a new BufferedImage which will be filtered. If the destination BufferedImage is
     *null, a new BufferedImage is returned, if not it must have the same height and width
     *as the source-BufferedImage. If not a new BufferedImage is returned.</p>
     * @param source the BufferedImage to be filtered,
     * @param dest the Destination-BufferedImage with the same height and width as Source,
     *and which may be null.
     * @return the filtered BufferedImage.
     * @see ImageCorrector
     * @see CorrectorSlider#setValue
     */
    public BufferedImage filter(BufferedImage source, BufferedImage dest){
	short[] contrast = new short[256];
	if(value != 0f){
	    for(int i=0; i<256; i++) {
		if( i> 127)
		    contrast[i] = (short)(i + ((i-127) * 0.01f * value));
		else if(i < 128)
		    contrast[i] = (short)(i - ((128 - i) * 0.01f * value));
		if(contrast[i] > 255) contrast[i] = 255;
		if(contrast[i] < 0) contrast[i] = 0;
	    }
	}
	else
	    return source;
	short[][] allContrast = new short[][]{contrast, contrast, contrast, contrast};
	BufferedImageOp contrastOp = new LookupOp(
						new ShortLookupTable(0, allContrast), null);
	BufferedImage img = getRedrawnImage(source);
	if(dest == null){
	    return  contrastOp.filter(img, null);
	}
	else {
	    if((dest.getHeight()== img.getHeight()) && 
	       (dest.getWidth() == img.getWidth())){
		return contrastOp.filter(img, dest);
	    }
	    else{
		return contrastOp.filter(img, null);
	    }
	}
    }
}
