package cdox.util.image;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * This class is the superclass of all the imageCorrection-classes. 
 *At the Time of this writing there are five implementing subclasses: <ul>
 *<li>Brightness</li>
 *<li>Contrast</li>
 *<li>GammaCorrection</li>
 *<li>Sharpen</li>
 *<li>Soften</li>
 *</ul>
 *It has the possibility to be used with a JSlider, or without. If you use this class
 *without the JSlider (e.g call the supclass-constuctor with the float-value) you should
 *use the setValue(float) method to update the value which is used in the filterMethod of
 *the supclasses. No value update no filter usage :-). If you want to use this JSlider you
 *should use the other constructor, and you must supply an Observer. Which will be called
 *when the JSlider has been moved. To retreive the container which contains the JSlider
 *call the getSliderPane-method.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 22nd 2002
 * @see #getSliderPane
 * @see Brightness
 * @see Contrast
 * @see GammaCorrection
 * @see Sharpen
 * @see Blur
 */
public abstract class CorrectorSlider extends Observable implements ImageCorrector, ChangeListener{
    /**
     *The slider which sets the value
     */
    protected JSlider corSlider;
    /**
     *The panel containing the slider
     */
    protected JPanel sliderPane;
    /**
     *The value used in the filter method
     */
    protected float value = 0f;
    /**
     *The current filterClass.
     */
    protected ImageCorrector imageCorrector;
    /**
     *The Observer to be called when the slider is dragged.
     */
    protected Component obs;
    
    /**
     *Constructs a CorrectorSlider without the JSlider. The sliderPane is initialized to
     *be used as an ImageObserver in the getRedrawnImage- Method. Use the setValue(float
     *val) method to set the current value.
     *@param val The initialization value used in the filter of the subclasses.
     *@see #getRedrawnImage
     */
    protected CorrectorSlider(float val){
	value = val;
	sliderPane = new JPanel();
    }

    /**
     *Constructs a new CorrectorSlider with the JSlider which is used to set the value to
     *be used in the sub-classes-filter method. The JSlider is used initialized with the
     *values given. The titledBorder of its container is set with the given paneTitle. The
     *Observer is registered to this Class, and is updated when the slider has been moved.
     *@param obs the Observer to be notified when the JSlider has been moved,
     *@param paneTitle the title of the titledBorder of the container containing the JSlider,
     *@param minValue the minimum value of the JSlider,
     *@param maxValue the maximum Value of the JSlider,
     *@param startValue the start value of the JSlider,
     *@param spacing the value between the spacings drawn on the JSlider,
     *@param labelSpacing the value between the spacing to draw a text,
     *@param grid true if snaptogrid false otherwhise.
     *@see #stateChanged
     */
    protected CorrectorSlider(Observer obs,
			      String paneTitle, 
			      int minValue, int maxValue, int startValue, int spacing, 
			      int labelSpacing,
			      boolean grid){
	addObserver(obs);
	this.obs = (Component)obs;
	sliderPane = new JPanel();
	sliderPane.setBorder(BorderFactory.createTitledBorder(paneTitle));
	corSlider = new JSlider(minValue, maxValue, startValue);
	corSlider.setMajorTickSpacing(spacing);
	corSlider.setPaintTicks(true);
	corSlider.setSnapToTicks(grid);
 	if(labelSpacing == -1){ //for gamma only
	    Hashtable gammaLabels = new Hashtable();
 	    gammaLabels.put(new Integer(0),new JLabel("0"));
	    gammaLabels.put(new Integer(100),new JLabel("1.00"));
	    gammaLabels.put(new Integer(200),new JLabel("2.00"));
	    gammaLabels.put(new Integer(300),new JLabel("3.00"));
	    corSlider.setLabelTable(gammaLabels);
 	}
	else{
	    corSlider.setLabelTable(corSlider.createStandardLabels(labelSpacing));
	}
	corSlider.setPaintLabels(true);
	corSlider.addChangeListener(this);
	sliderPane.add(corSlider);
    }

    /**
     *Called when the JSlider has been moved. If the JSlider has been moved an is not
     *adjusting now, the Observer is called and value is set. If the user is adjusting the
     *JSlider, the ToolTipText is set to the current value of the JSlider (doesn't work so
     *fine yet).
     *@param ce the ChangeEvent representing a movement of the JSlider.
     */
    public void stateChanged(ChangeEvent ce){ 
	if(!corSlider.getValueIsAdjusting()){
	    value=(float)corSlider.getValue();
	    setChanged();
	    notifyObservers(imageCorrector);
	}
	else{
	    corSlider.setToolTipText(new Integer(corSlider.getValue()).toString());
	}
    }
    
    /**
     *Returns the container containing the JSlider which can be added to your Dialog,
     *JFrame or every other container.
     *@return the JPanel containing the Slider.
     */
    public JPanel getSliderPane(){
	return sliderPane;
    }

    /**
     *Returns the current value with which the current filter is working.
     *@return the current value.
     */
    public int getValue(){
	return (int)value;
    }

    /**
     *This method is used to set the value which is used in the current filter, you should
     *use this method, if you have chosen to define your own JSlider-classes.
     *@param val the value to set the filters-working-value.
     */
    public void setValue(float val){
	value = val;
    }

    /**
     *This method draws the given BufferedImage on a new BufferedImage and returns it. The
     *returned method is one of TYPE_INT_ARGB which means it has an Alpha chanel. This
     *method insures that all images are filterable (JDK-BUG?). And that transperable gif
     *images are still transparent after filtering.
     *@param bi the BufferedImage to be redrawn,
     *@return the redrawn TYPE_INT_ARGB BufferedImage.
     */
    protected BufferedImage getRedrawnImage(BufferedImage bi){
	BufferedImage img = new BufferedImage(bi.getWidth(), bi.getHeight(), 
					      BufferedImage.TYPE_INT_ARGB);
	Graphics2D g2=img.createGraphics();
	g2.drawImage(bi, null, sliderPane);
	bi=null;
	System.gc();
	return img;
    }
}
