package cdox.util.image;

import java.util.Observer;
import java.awt.image.*;
import java.awt.*;
/**
 * This class highers or loweres the gamma of a BufferedImage. It uses a lookupTable to
 * do so. To use this class you can call one of its two constructors. If you've designed
 * your own Jslider classes you can use the constructor which takes a float-value. You than
 * can use the setValue(float val) method to set the value to work with. And than call the
 * filter method to filter the BufferedImage.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 22nd 2002
 * @see CorrectorSlider
 */
public class GammaCorrection extends CorrectorSlider implements ImageCorrector{
    
     /**
      *This constructor creates a new GammaCorrection Object, with a JSlider in a JPanel with a
      *titledBorder set with the name paneTitle. The values of the JSlider are standardly
      *min=-0, max = 300, initValue=100, spacings=100, textatSpacing=-1,(constructs a new Hashmap)
      *graptogrid=false. The Observer is called when the JSlider is moved. This Observer
      *could be a preview pane or anything else. 
      *@param obs The Observer to handle the BufferedImage.
      *@param paneTitle the title of the titledBorder of the JPane surrounding the JSlider.
      *@see CorrectorSlider#CorrectorSlider(Observer, String, int, int, int, int, int, boolean).
      */
    public GammaCorrection(Observer obs, String paneTitle){
	super(obs, paneTitle, 0,300,100,100,-1,false);
	imageCorrector = this;
    }
    /**
     *This constructor creates a new GammaCorrection Object with its initial-Value set to val. It
     *has no JSlider and the value of this class (to work with) should be set by calling
     *setValue(float val).
     *@param val the initial float value.
     *@see CorrectorSlider#CorrectorSlider(float)
     *@see CorrectorSlider#setValue
     */
    public GammaCorrection(float val){
	super(val);
    }
    /**
     * <p>The actual filtering is done in this method defined in ImageCorrector. It uses a
     *ShortLookupTable to make the gamma higher or lower for all the four chanels (Red,
     *Green, Blue, Alpha). First the short-array is filled with the values, the
     *colornumber = i(between 0-255)are powered to(1/current-value)<I>(set by the JSlider or
     *setValue(float val))</I> which is afterwards multiplied with 255(white) this ensures
     *that all dark colors (small i) are upped and the light colors(large i) are upped a
     *little (to none). After that this short-array is copied in a
     *2-dimensional-short-array, which represent the color-chanels. With BufferedImageOp a
     *new LookupOp is done, which will be used to filter the source BufferedImage.</p>    
     *<p>To make sure the source-BufferedImage can be "gamma-ed" the image is redrawn on
     *a new BufferedImage which will be filtered. If the destination BufferedImage is
     *null, a new BufferedImage is returned, if not it must have the same height and width
     *as the source-BufferedImage. If not a new BufferedImage is returned.</p>
     * @param source the BufferedImage to be filtered,
     * @param dest the Destination-BufferedImage with the same height and width as Source,
     *and which may be null.
     * @return the filtered BufferedImage.
     * @see ImageCorrector
     * @see CorrectorSlider#setValue
     */
    public BufferedImage filter(BufferedImage source, BufferedImage dest){
	short[] gamma = new short[256];
	for(int i=0; i<256; i++) {
	    gamma[i] = (short)Math.round((255 * Math.pow(((double)i / 255), (double)1/(value/100))));
	}
	short[][] allGamma = new short[][]{gamma, gamma, gamma, gamma};
	BufferedImageOp gammaOp = new LookupOp(
						new ShortLookupTable(0, allGamma), null);
	BufferedImage img = getRedrawnImage(source);
	if(dest == null){
	    return  gammaOp.filter(img, null);
	}
	else {
	    if((dest.getHeight()== img.getHeight()) && 
	       (dest.getWidth() == img.getWidth())){
		return gammaOp.filter(img, dest);
	    }
	    else{
		return gammaOp.filter(img, null);
	    }
	}
    }
}
