package cdox.util.image;

import java.awt.*;
import java.awt.image.*;
import java.util.Observer;
/**
 *This class sharpens a BufferedImage. It uses a kernel and ConvolOp to do so. To use this
 *class you can call one of its two constructors. If you've designed your own Jslider
 *classes you can use the constructor which takes a float-value. You than can use the
 *setValue(float val) method to set the value to work with. And than call the filter
 *method to filter the BufferedImage.If value == 0f the original (source-image) will be
 *returned.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version 16 April 2002
 * @see CorrectorSlider
 */
public class Sharpen extends CorrectorSlider implements ImageCorrector{
    
    /**
     *The float kernel, the middle is the current pixel.
     */
    private float[] sharpKernel = {0f, -1f, 0f,
				   -1f, 5f, -1f,
				   0f, -1f, 0f};

    /**
     *The ConvolveOp operation.
     */
    private BufferedImageOp sharpen;
    /**
     *Needs to be done if the image should be recognizable after one iteration
     */
    private Blur blur;
    /**
     *This constructor creates a new Sharpen Object, with a JSlider in a JPanel with a
     *titledBorder set with the name paneTitle. The values of the JSlider are standardly
     *min=0, max = 6, initValue=0, spacings=1, textatSpacing=1,
     *graptogrid=true. The Observer is called when the JSlider is moved. This Observer
     *could be a preview pane or anything else. 
     *@param obs The Observer to handle the BufferedImage.
     *@param paneTitle the title of the titledBorder of the JPane surrounding the JSlider.
     *@see CorrectorSlider#CorrectorSlider(Observer, String, int, int, int, int, int, boolean).
     */
    public Sharpen(Observer obs, String paneTitle){
	super(obs, paneTitle, 0,6,0,1,1, true);
	imageCorrector = this;
	sharpen = new ConvolveOp(new Kernel(3, 3, sharpKernel));
	blur = new Blur(1f);
    }
    /**
     *This constructor creates a new Sharpen Object with its initial-Value set to val. It
     *has no JSlider and the value of this class (to work with) should be set by calling
     *setValue(float val).
     *@param val the initial float value.
     *@see CorrectorSlider#CorrectorSlider(float)
     *@see CorrectorSlider#setValue
     */
    public Sharpen(float val){
	super(val);
	sharpen = new ConvolveOp(new Kernel(3, 3, sharpKernel));
	blur = new Blur(1f);
    }
    /**
     *<p>The actual filtering is done in this method defined in ImageCorrector. It uses a
     *Kernel and a ConvolveOp to make the BufferedImage look more sharpen for all the four
     *chanels (Red,Green, Blue, Alpha). </p>
     *<p>The Kernel is defined as a "3x3"-Matrix of floats with following form: <ul>
     *<li>0.0, -1.0, 0.0</li>
     *<li>-1.0, 5.0, -1.0</li>
     *<li>0.0, -1.0, 0.0</li></ul>.
     *This Kernel is used in a ConvolveOp Object. For each value of "value" *<I>(set by
     *the JSlider or setValue(float val))</I> this method first blurs the source image,
     *and than applies the ConvolveOp to sharpen it. If the blurring not is done, the
     *image would be irrecognizable for values >1. It causes this method to be very slow
     *for larger values (>3) though, life's a bitch. </p>
     *<p>To make sure the source-BufferedImage can be "sharpened" the image is redrawn on
     *a new BufferedImage which will be filtered. If the destination BufferedImage is
     *null, a new BufferedImage is returned, if not it must have the same height and width
     *as the source-BufferedImage. If not a new BufferedImage is returned.</p>
     * @param source the BufferedImage to be filtered,
     * @param dest the Destination-BufferedImage with the same height and width as Source,
     *and which may be null.
     * @return the filtered BufferedImage.
     * @see Blur
     * @see ImageCorrector
     * @see CorrectorSlider#setValue
     */
    public BufferedImage filter(BufferedImage source, BufferedImage dest){
	if(value==0f || value > 6.0f){
	    return source;
	}
	BufferedImage img = getRedrawnImage(source);
	BufferedImage filteredImage=null;
	if(dest == null){
	    filteredImage = getRedrawnImage(img);
	    return doFiltering(img, filteredImage);
	}
	else {
	    if((dest.getHeight()== img.getHeight()) && 
	       (dest.getWidth() == img.getWidth())){
		dest = getRedrawnImage(dest);
		return doFiltering(img, dest);
	    }
	    else{
		filteredImage = getRedrawnImage(img);
		return doFiltering(img, filteredImage);
	    }
	}
    }
    /**
     *Here the actual filtering is done, this method iterates as often as value is
     *set. Which sharpens the image more and more.
     *@param source the to filter image,
     *@param dest the filtered image.
     *@return the filtered BufferedImage given in dest.
     *@see #filter
     */
    private BufferedImage doFiltering(BufferedImage source, BufferedImage dest){
	for(int i= 0; i<value;i++){
	    source=blur.filter(source, null);
	    dest = sharpen.filter(source, dest);
	    source = getRedrawnImage(dest);
	}
	return dest;
    }
}
